@@ -328,6 +328,7 @@
       [newOccurence autorelease];
       [newOccurence removeAllRecurrenceRules];
       [newOccurence removeAllExceptionRules];
+      [newOccurence removeAllRecurrenceDates];
       [newOccurence removeAllExceptionDates];
 
       // It is important to set the organizer as some DAV clients (iCal
@@ -686,6 +687,7 @@
 
 - (NSException *) saveComponent: (iCalRepeatableEntityObject *) newObject
 {
+  [newObject setLastModified: [NSCalendarDate calendarDate]];
   return [self saveCalendar: [newObject parent]];
 }
 
@@ -804,15 +806,17 @@
 	  /* sender */
 	  shortSenderEmail = [[object organizer] rfc822Email];
 	  if ([shortSenderEmail length])
-            {
-              senderEmail = [[object organizer] mailAddress];
-            }
+            senderEmail = [[object organizer] mailAddress];
           else
             {
               shortSenderEmail = [[previousObject organizer] rfc822Email];
               senderEmail = [[previousObject organizer] mailAddress];
             }
 
+          // No organizer, grab the event's owner
+          if (![senderEmail length])
+            senderEmail = shortSenderEmail = [[ownerUser defaultIdentity] objectForKey: @"email"];
+
           /* calendar part */
           eventBodyPart = [self _bodyPartForICalObject: object];
 
@@ -1160,7 +1164,8 @@
                        inContext: context];
 	}
       
-      if ([[self container] notifyUserOnPersonalModifications])
+      if ([[self container] notifyUserOnPersonalModifications] &&
+          [[self container] notifiedUserOnPersonalModifications])
 	{
 	  id o;
 
