@@ -1,6 +1,6 @@
 /* NGVCard+SOGo.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2016 Inverse inc.
+ * Copyright (C) 2009-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
  */
 
 
+#import <Foundation/NSValue.h>
 #import <Foundation/NSTimeZone.h>
 
 #import <NGExtensions/NGBase64Coding.h>
@@ -193,24 +194,27 @@ convention:
   NSEnumerator *list;
   CardElement *element;
 
-  // value is either an array or a string
-  if ([value isKindOfClass: [NSString class]])
-    allValues = [NSArray arrayWithObject: value];
-  else
-    allValues = value;
-
-  // Add all values as separate elements
-  list = [allValues objectEnumerator];
-  while ((value = [list nextObject]))
+  if ([value isNotNull])
     {
-      if ([type length])
-        element = [CardElement simpleElementWithTag: elementTag
-                                         singleType: type
-                                              value: value];
+      // value is either an array or a string
+      if ([value isKindOfClass: [NSString class]])
+        allValues = [NSArray arrayWithObject: value];
       else
-        element = [CardElement simpleElementWithTag: elementTag
-                                              value: value];
-      [self addChild: element];
+        allValues = value;
+
+      // Add all values as separate elements
+      list = [allValues objectEnumerator];
+      while ((value = [list nextObject]))
+        {
+          if ([type length])
+            element = [CardElement simpleElementWithTag: elementTag
+                                             singleType: type
+                                                  value: value];
+          else
+            element = [CardElement simpleElementWithTag: elementTag
+                                                  value: value];
+          [self addChild: element];
+        }
     }
 }
 
@@ -325,18 +329,18 @@ convention:
   [element setValues: [ldifRecord objectForKey: @"c"]
 	     atIndex: 6 forKey: @""];
 
-  ou = [ldifRecord objectForKey: @"ou"];
-  if ([ou isKindOfClass: [NSArray class]])
-    units = [NSMutableArray arrayWithArray: (NSArray *)ou];
-  else if (ou)
-    units = [NSMutableArray arrayWithObject: ou];
-  else
-    units = [NSMutableArray array];
   o = [ldifRecord objectForKey: @"o"];
   if ([o isKindOfClass: [NSArray class]])
-    [units addObjectsFromArray: (NSArray *)o];
+    units = [NSMutableArray arrayWithArray: (NSArray *)o];
   else if (o)
-    [units addObject: o];
+    units = [NSMutableArray arrayWithObject: o];
+  else
+    units = [NSMutableArray array];
+  ou = [ldifRecord objectForKey: @"ou"];
+  if ([ou isKindOfClass: [NSArray class]])
+    [units addObjectsFromArray: (NSArray *)ou];
+  else if (ou)
+    [units addObject: ou];
   [self setOrganizations: units];
 
   [self _setPhoneValues: ldifRecord];
@@ -380,7 +384,7 @@ convention:
   o = [ldifRecord objectForKey: @"description"];
   if ([o isKindOfClass: [NSArray class]])
     [self setNotes: o];
-  else
+  else if ([o isNotNull])
     [self setNote: o];
 
   o = [ldifRecord objectForKey: @"vcardcategories"];
@@ -389,7 +393,7 @@ convention:
   // string (from a LDIF import) as the value here.
   if ([o isKindOfClass: [NSArray class]])
     [self setCategories: o];
-  else
+  else if ([o isNotNull])
     [self setCategories: [o componentsSeparatedByString: @","]];
 
   // Photo
@@ -1029,6 +1033,11 @@ convention:
     [fields setObject: [NSNull null] forKey: @"c_categories"];
   [fields setObject: @"vcard" forKey: @"c_component"];
 
+  // S/MIME certificate
+  element = [self uniqueChildWithTag: @"key"];
+  [fields setObject: [NSNumber numberWithInt: (element && ![element isVoid])]
+             forKey: @"c_hascertificate"];
+
   return fields;
 }
 
