@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2016 Inverse inc.
+  Copyright (C) 2007-2018 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -41,6 +41,7 @@
 #import <NGMail/NGMailAddressParser.h>
 #import <NGMail/NGMimeMessage.h>
 #import <NGMail/NGMimeMessageGenerator.h>
+#import <NGMail/NGMimeMessageParser.h>
 #import <NGMime/NGMimeBodyPart.h>
 #import <NGMime/NGMimeFileData.h>
 #import <NGMime/NGMimeMultipartBody.h>
@@ -65,6 +66,7 @@
 #import <Contacts/SOGoContactGCSEntry.h>
 
 #import "NSData+Mail.h"
+#import "NSData+SMIME.h"
 #import "NSString+Mail.h"
 #import "SOGoDraftsFolder.h"
 #import "SOGoMailAccount.h"
@@ -120,13 +122,16 @@ static NSString    *userAgent      = nil;
     {
       sourceIMAP4ID = -1;
       IMAP4ID = -1;
-      headers = [NSMutableDictionary new];
+      headers = [[NSMutableDictionary alloc] init];
+      certificates = [[NSMutableDictionary alloc] init];
       text = @"";
       path = nil;
       sourceURL = nil;
       sourceFlag = nil;
       inReplyTo = nil;
       isHTML = NO;
+      sign = NO;
+      encrypt = NO;
     }
 
   return self;
@@ -135,6 +140,7 @@ static NSString    *userAgent      = nil;
 - (void) dealloc
 {
   [headers release];
+  [certificates release];
   [text release];
   [path release];
   [sourceURL release];
@@ -317,6 +323,25 @@ static NSString    *userAgent      = nil;
   return isHTML;
 }
 
+- (void) setSign: (BOOL) aBool
+{
+  sign = aBool;
+}
+- (BOOL) sign
+{
+  return sign;
+}
+
+- (void) setEncrypt: (BOOL) aBool
+{
+  encrypt = aBool;
+}
+
+- (BOOL) encrypt
+{
+  return encrypt;
+}
+
 - (NSString *) inReplyTo
 {
   return inReplyTo;
@@ -400,7 +425,7 @@ static NSString    *userAgent      = nil;
 	  [infos setObject: sourceFolder forKey: @"sourceFolder"];
 	}
 
-      if ([infos writeToFile: [self infoPath] atomically:YES])
+      if ([infos writeToFile: [self infoPath]  atomically: YES])
 	error = nil;
       else
 	{
@@ -536,7 +561,7 @@ static NSString    *userAgent      = nil;
   id result;
 
   error = nil;
-  message = [self mimeMessageAsData];
+  message = [self mimeMessageForRecipient: nil];
 
   if (!message)
     {
@@ -730,7 +755,7 @@ static NSString    *userAgent      = nil;
   /* CC processing if we reply-to-all: - we add all 'to' and 'cc' fields */
   if (_replyToAll)
     {
-      to = [NSMutableArray new];
+      to = [[NSMutableArray alloc] init];
 
       [addrs setArray: [_envelope to]];
       [self _purgeRecipients: allRecipients
@@ -754,9 +779,10 @@ static NSString    *userAgent      = nil;
 //
 - (void) _fetchAttachmentsFromMail: (SOGoMailObject *) sourceMail
 {
-  unsigned int max, count;
-  NSArray *attachments;
   NSDictionary *currentInfo;
+  NSArray *attachments;
+
+  unsigned int max, count;
 
   attachments = [sourceMail fetchFileAttachments];
   max = [attachments count];
@@ -768,6 +794,76 @@ static NSString    *userAgent      = nil;
     }
 }
 
+//
+//
+//
+- (void) _fileAttachmentsFromPart: (id) thePart
+{
+  // Small hack to avoid SOPE's stupid behavior to wrap a multipart
+  // object in a NGMimeBodyPart.
+  if ([thePart isKindOfClass: [NGMimeBodyPart class]] &&
+      [[[thePart contentType] type] isEqualToString: @"multipart"])
+     thePart = [thePart body];
+
+  if ([thePart isKindOfClass: [NGMimeBodyPart class]])
+    {
+      NSString *filename, *mimeType;
+      id body;
+
+      mimeType = [[thePart contentType] stringValue];
+      body = [thePart body];
+      filename = [(NGMimeContentDispositionHeaderField *)[thePart headerForKey: @"content-disposition"] filename];
+
+      if (!filename)
+        filename = [mimeType asPreferredFilenameUsingPath: nil];
+
+      if (filename)
+        {
+          NSDictionary *currentInfo;
+
+          currentInfo = [NSDictionary dictionaryWithObjectsAndKeys:
+                                            filename, @"filename",
+                                      mimeType, @"mimetype",
+                                      nil];
+          [self saveAttachment: body
+                  withMetadata: currentInfo];
+        }
+    }
+  else if ([thePart isKindOfClass: [NGMimeMultipartBody class]])
+    {
+      NSArray *parts;
+      int i;
+
+      parts = [thePart parts];
+      for (i = 0; i < [parts count]; i++)
+        {
+          [self _fileAttachmentsFromPart: [parts objectAtIndex: i]];
+        }
+    }
+}
+
+
+//
+//
+//
+- (void) _fetchAttachmentsFromEncryptedMail: (SOGoMailObject *) sourceMail
+{
+  NSData *certificate;
+
+  certificate = [[self mailAccountFolder] certificate];
+
+  // If we got a user certificate, let's use it. Otherwise we fallback we
+  // don't try to get any attachments from the encrypted content
+  if (certificate)
+    {
+      NGMimeMessage *m;
+
+      m = [[sourceMail content] messageFromEncryptedDataAndCertificate: certificate];
+      [self _fileAttachmentsFromPart: [m body]];
+    }
+}
+
+
 //
 //
 //
@@ -835,6 +931,7 @@ static NSString    *userAgent      = nil;
 {
   BOOL fromSentMailbox;
   NSString *msgID;
+  NSMutableArray *addresses;
   NSMutableDictionary *info;
   NGImap4Envelope *sourceEnvelope;
 
@@ -852,6 +949,12 @@ static NSString    *userAgent      = nil;
   msgID = [sourceEnvelope messageID];
   if ([msgID length] > 0)
     [self setInReplyTo: msgID];
+
+  addresses = [NSMutableArray array];
+  [self _addEMailsOfAddresses: [sourceEnvelope to] toArray: addresses];
+  if ([addresses count])
+    [info setObject: [addresses objectAtIndex: 0] forKey: @"from"];
+
   [self setText: [sourceMail contentForReply]];
   [self setHeaders: info];
   [self setSourceURL: [sourceMail imap4URLString]];
@@ -887,7 +990,10 @@ static NSString    *userAgent      = nil;
   if ([[ud mailMessageForwarding] isEqualToString: @"inline"])
     {
       [self setText: [sourceMail contentForInlineForward]];
-      [self _fetchAttachmentsFromMail: sourceMail];
+      if ([sourceMail isEncrypted])
+        [self _fetchAttachmentsFromEncryptedMail: sourceMail];
+      else
+        [self _fetchAttachmentsFromMail: sourceMail];
     }
   else
     {
@@ -984,21 +1090,24 @@ static NSString    *userAgent      = nil;
 {
   NSString *p, *pmime, *name, *mimeType;
 
-  if (![_attach isNotNull]) {
-    return [NSException exceptionWithHTTPStatus:400 /* Bad Request */
-			reason: @"Missing attachment content!"];
-  }
+  if (![_attach isNotNull])
+    {
+      return [NSException exceptionWithHTTPStatus: 400 /* Bad Request */
+                                           reason: @"Missing attachment content!"];
+    }
 
-  if (![self _ensureDraftFolderPath]) {
-    return [NSException exceptionWithHTTPStatus:500 /* Server Error */
-			reason: @"Could not create folder for draft!"];
-  }
+  if (![self _ensureDraftFolderPath])
+    {
+      return [NSException exceptionWithHTTPStatus: 500 /* Server Error */
+                                           reason: @"Could not create folder for draft!"];
+    }
 
   name = [[metadata objectForKey: @"filename"] asSafeFilename];
   p = [self pathToAttachmentWithName: name];
+
   if (![_attach writeToFile: p atomically: YES])
     {
-      return [NSException exceptionWithHTTPStatus:500 /* Server Error */
+      return [NSException exceptionWithHTTPStatus: 500 /* Server Error */
 			  reason: @"Could not write attachment to draft!"];
     }
 
@@ -1006,7 +1115,7 @@ static NSString    *userAgent      = nil;
   if ([mimeType length] > 0)
     {
       pmime = [self pathToAttachmentWithName: [NSString stringWithFormat: @".%@.mime", name]];
-      if (![[mimeType dataUsingEncoding: NSUTF8StringEncoding] writeToFile: pmime atomically: YES])
+      if (![[mimeType dataUsingEncoding: NSUTF8StringEncoding] writeToFile: pmime  atomically: YES])
         {
           [[NSFileManager defaultManager] removeFileAtPath: p  handler: nil];
           return [NSException exceptionWithHTTPStatus: 500 /* Server Error */
@@ -1089,7 +1198,7 @@ static NSString    *userAgent      = nil;
   NGMimeMessage *message;
   id body;
 
-  message = [[[NGMimeMessage alloc] initWithHeader:map] autorelease];
+  message = [[[NGMimeMessage alloc] initWithHeader: map] autorelease];
 
   if (!isHTML)
     {
@@ -1256,8 +1365,8 @@ static NSString    *userAgent      = nil;
 	content = [content dataByEncodingBase64];
         [map setObject: @"base64" forKey: @"content-transfer-encoding"];
       }
-    [map setObject:[NSNumber numberWithInt:[content length]]
-	 forKey: @"content-length"];
+    [map setObject: [NSNumber numberWithInt: [content length]]
+            forKey: @"content-length"];
 
     /* Note: the -init method will create a temporary file! */
     body = [[NGMimeFileData alloc] initWithBytes:[content bytes]
@@ -1307,7 +1416,7 @@ static NSString    *userAgent      = nil;
 //
 //
 //
-- (NGMimeBodyPart *) mimeMultipartAlternative
+- (NGMimeBodyPart *) mimeMultipartAlternative: (NSArray *) extractedBodyParts
 {
   NGMimeMultipartBody *textParts;
   NGMutableHashMap *header;
@@ -1323,8 +1432,35 @@ static NSString    *userAgent      = nil;
   // Get the text part from it and add it
   [textParts addBodyPart: [self plainTextBodyPartForText]];
 
-  // Add the HTML part
-  [textParts addBodyPart: [self bodyPartForText]];
+  if ([extractedBodyParts count])
+    {
+      // Create a multipart/related part and add this.
+      // We have inline image to avoid Thunderbird bug #61815 (https://bugzilla.mozilla.org/show_bug.cgi?id=61815)
+      NGMutableHashMap *relatedHeader;
+      NGMimeBodyPart *relatedPart;
+      NGMimeMultipartBody *relatedParts;
+      int i;
+
+      relatedHeader = [NGMutableHashMap hashMap];
+      [relatedHeader addObject: MultiRelatedType forKey: @"content-type"];
+      relatedPart = [NGMimeBodyPart bodyPartWithHeader: relatedHeader];
+      relatedParts = [[NGMimeMultipartBody alloc] initWithPart: relatedPart];
+
+      [relatedParts addBodyPart: [self bodyPartForText]];
+
+      for (i = 0; i < [extractedBodyParts count]; i++)
+        {
+          [relatedParts addBodyPart: [extractedBodyParts objectAtIndex: i]];
+        }
+
+      [relatedPart setBody: relatedParts];
+      [textParts addBodyPart: relatedPart];
+    }
+  else
+    {
+      // Add the HTML part
+      [textParts addBodyPart: [self bodyPartForText]];
+    }
 
   [part setBody: textParts];
   RELEASE(textParts);
@@ -1336,7 +1472,9 @@ static NSString    *userAgent      = nil;
 //
 //
 - (NGMimeMessage *) mimeMultiPartMessageWithHeaderMap: (NGMutableHashMap *) map
-					 andBodyParts: (NSArray *) _bodyParts
+                                   extractedBodyParts: (NSArray *) extractedBodyParts
+  					 andBodyParts: (NSArray *) _bodyParts
+                                             bodyOnly: (BOOL) _bodyOnly
 {
   NGMimeMessage       *message;
   NGMimeMultipartBody *mBody;
@@ -1352,7 +1490,7 @@ static NSString    *userAgent      = nil;
   if (!isHTML)
     part = [self bodyPartForText];
   else
-    part = [self mimeMultipartAlternative];
+    part = [self mimeMultipartAlternative: extractedBodyParts];
 
   [mBody addBodyPart: part];
 
@@ -1550,31 +1688,27 @@ static NSString    *userAgent      = nil;
 - (NGMimeMessage *) mimeMessageWithHeaders: (NSDictionary *) _headers
 				 excluding: (NSArray *) _exclude
                           extractingImages: (BOOL) _extractImages
+                                  bodyOnly: (BOOL) _bodyOnly
 {
-  NSMutableArray *bodyParts;
+  NSMutableArray *extractedBodyParts;
   NGMimeMessage *message;
   NSArray *allBodyParts;
   NGMutableHashMap *map;
   NSString *newText;
 
-  BOOL has_inline_images;
-
   message = nil;
-  has_inline_images = NO;
-  bodyParts = [NSMutableArray array];
+  extractedBodyParts = [NSMutableArray array];
 
   if (_extractImages)
     {
-      newText = [text htmlByExtractingImages: bodyParts];
-      if ([bodyParts count])
-        {
-          [self setText: newText];
-          has_inline_images = YES;
-        }
+      newText = [text htmlByExtractingImages: extractedBodyParts];
+      if ([extractedBodyParts count])
+        [self setText: newText];
     }
 
   map = [self mimeHeaderMapWithHeaders: _headers
                              excluding: _exclude];
+
   if (map)
     {
       //[self debugWithFormat: @"MIME Envelope: %@", map];
@@ -1583,26 +1717,19 @@ static NSString    *userAgent      = nil;
       if (!allBodyParts)
 	return nil;
 
-      [bodyParts addObjectsFromArray: allBodyParts];
-      
       //[self debugWithFormat: @"attachments: %@", bodyParts];
 
-      if ([bodyParts count] == 0)
-        /* no attachments */
-        message = [self mimeMessageForContentWithHeaderMap: map];
+      if ([extractedBodyParts count] == 0 && [allBodyParts count] == 0)
+        {
+          // no attachment
+          message = [self mimeMessageForContentWithHeaderMap: (_bodyOnly ? nil : map)];
+        }
       else
         {
-          // attachments, create multipart/mixed or multipart/related if
-          // we have inline image to avoid Thunderbird bug #61815 (https://bugzilla.mozilla.org/show_bug.cgi?id=61815)
-          if (has_inline_images)
-            {
-              [map removeAllObjectsForKey: @"content-type"];
-              [map addObject: MultiRelatedType forKey: @"content-type"];
-            }
-
-          message = [self mimeMultiPartMessageWithHeaderMap: map
-                                               andBodyParts: bodyParts];
-
+          message = [self mimeMultiPartMessageWithHeaderMap: (_bodyOnly ? [NGMutableHashMap hashMap] : map)
+                                         extractedBodyParts: extractedBodyParts
+                                               andBodyParts: allBodyParts
+                                                   bodyOnly: _bodyOnly];
           //[self debugWithFormat: @"message: %@", message];
         }
     }
@@ -1613,33 +1740,73 @@ static NSString    *userAgent      = nil;
 //
 // Return a NGMimeMessage object with inline HTML images (<img src=data>) extracted as attachments (<img src=cid>).
 //
-- (NGMimeMessage *) mimeMessage
+- (NSData *) mimeMessageForRecipient: (NSString *) theRecipient
 {
-  return [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: YES];
-}
+  NGMimeMessageGenerator *generator, *partGenerator;
+  NSData *certificate, *content;
+  NGMutableHashMap *hashMap;
+  NGMimeMessage *message;
+  NSMutableData *d;
 
-//
-// Return a NSData object of the message with no alteration.
-//
-- (NSData *) mimeMessageAsData
-{
-  NGMimeMessageGenerator *generator;
-  NGMimeMessage *mimeMessage;
-  NSData *message;
-
-  generator = [NGMimeMessageGenerator new];
-  mimeMessage = [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: NO];
-
-  if (!mimeMessage)
+  // Nothing to sign or encrypt, let's generate the message and return immediately
+  if (![self sign] && ![self encrypt])
     {
-      [generator release];
-      return nil;
+      generator = [[[NGMimeMessageGenerator alloc] init] autorelease];
+      return [generator generateMimeFromPart: [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: YES  bodyOnly: NO]];
     }
 
-  message = [generator generateMimeFromPart: mimeMessage];
-  [generator release];
+  // We'll sign and/or encrypt our message. Let's generate the actual body of the message to work with
+  partGenerator = [[[NGMimePartGenerator alloc] init] autorelease];
+  content = [partGenerator generateMimeFromPart: [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: YES  bodyOnly: YES]];
 
-  return message;
+  if ([self sign])
+    {
+      certificate = [[self mailAccountFolder] certificate];
+      content = [content signUsingCertificateAndKey: certificate];
+
+      if (!content)
+        return nil;
+
+      if (![self encrypt])
+        goto finish_smime;
+    }
+
+  if ([self encrypt])
+    {
+      if (theRecipient)
+        {
+          SOGoContactFolders *contactFolders;
+
+          contactFolders = [[[context activeUser] homeFolderInContext: context]
+                                  lookupName: @"Contacts"
+                                   inContext: context
+                                     acquire: NO];
+          certificate = [[contactFolders certificateForEmail: theRecipient] convertPKCS7ToPEM];
+        }
+      else
+        certificate =  [[self mailAccountFolder] certificate];
+
+      // We check if we have a valid certificate. We can have nil here coming from [[self mailAccountFolder] certificate].
+      // This can happen if one sends an encrypted mail, but actually never uploaded
+      // a PKCS#12 file to SOGo for his/her own usage and we're trying to save an encrypted
+      // version of the message in the current user's Sent folder
+      if (certificate)
+        content = [content encryptUsingCertificate: certificate];
+    }
+
+ finish_smime:
+  // We got our mime part, let's add our mail headers
+  hashMap = [self mimeHeaderMapWithHeaders: nil
+                                 excluding: [NSArray arrayWithObjects: @"MIME-Version", @"Content-Type", @"Content-Transfer-Encoding", nil]];
+  message = [NGMimeMessage messageWithHeader: hashMap];
+  generator = [[[NGMimeMessageGenerator alloc] init] autorelease];
+  d = [NSMutableData dataWithData: [generator generateMimeFromPart: message]];
+  [d replaceBytesInRange: NSMakeRange([d length]-4, 4)
+                          withBytes: NULL
+                             length: 0];
+  [d appendData: content];
+
+  return d;
 }
 
 //
@@ -1687,23 +1854,60 @@ static NSString    *userAgent      = nil;
 //
 - (NSException *) sendMail
 {
+  SOGoContactFolders *contactFolders;
   SOGoUserDefaults *ud;
+  NSArray *recipients;
+  NSString *recipient;
+  int i;
 
   ud = [[context activeUser] userDefaults];
 
+  // If we are trying to sign an email but we don't have a S/MIME certificate for that
+  // IMAP account, we abort
+  if ([self sign] && ![[self mailAccountFolder] certificate])
+    {
+      return [NSException exceptionWithHTTPStatus: 500 /* server error */
+                                           reason: @"cannot sign email without certificate"]; 
+    }
+
+  // If we are encrypting emails, we must make sure that we have the certificate
+  // for all recipients otherwise we cannot, of course, encrypt the email.
+  if ([self encrypt])
+    {
+      NSData *certificate;
+
+      contactFolders = [[[context activeUser] homeFolderInContext: context]
+                               lookupName: @"Contacts"
+                                inContext: context
+                                  acquire: NO];
+      recipients = [self allBareRecipients];
+      for (i = 0; i < [recipients count]; i++)
+        {
+          recipient = [recipients objectAtIndex: i];
+
+          if ([[context activeUser] hasEmail: recipient])
+            certificate = [[self mailAccountFolder] certificate];
+          else
+            certificate = [contactFolders certificateForEmail: recipient];
+
+          if (!certificate)
+            return [NSException exceptionWithHTTPStatus: 500 /* server error */
+                                                 reason: @"cannot encrypt email without recipient certificate"];
+
+          [certificates setObject: certificate  forKey: recipient];
+        }
+    }
+
   if ([ud mailAddOutgoingAddresses])
     {
-      NSString *recipient, *emailAddress, *addressBook, *uid;
-      NSArray *matchingContacts, *recipients;
-      SOGoContactFolders *contactFolders;
+      NSString *emailAddress, *addressBook, *uid;
+      NSArray *matchingContacts;
       SOGoContactGCSEntry *newContact;
       NGMailAddress *parsedRecipient;
       NGMailAddressParser *parser;
       SOGoFolder <SOGoContactFolder> *folder;
       NGVCard *card;
 
-      int i;
-
       // Get all the addressbooks
       contactFolders = [[[context activeUser] homeFolderInContext: context]
                          lookupName: @"Contacts"
@@ -1737,13 +1941,13 @@ static NSString    *userAgent      = nil;
               [card addEmail: emailAddress types: nil];
               [card setFn: [parsedRecipient displayName]];
 
-              newContact = [SOGoContactGCSEntry objectWithName: uid
-                                                   inContainer: folder];
+              newContact = [SOGoContactGCSEntry objectWithName: uid  inContainer: folder];
               [newContact setIsNew: YES];
               [newContact saveComponent: card];
             }
         }
     }
+
   return [self sendMailAndCopyToSent: YES];
 }
 
@@ -1752,24 +1956,71 @@ static NSString    *userAgent      = nil;
 //
 - (NSException *) sendMailAndCopyToSent: (BOOL) copyToSent
 {
+  NSData *message, *messageForSent;
   SOGoMailFolder *sentFolder;
   SOGoDomainDefaults *dd;
   NSURL *sourceIMAP4URL;
   NSException *error;
-  NSData *message;
 
-  // We strip the BCC fields prior sending any mails
-  NGMimeMessageGenerator *generator;
-
-  generator = [[[NGMimeMessageGenerator alloc] init] autorelease];
-  message = [generator generateMimeFromPart: [self mimeMessage]];
   dd = [[context activeUser] domainDefaults];
-  error = [[SOGoMailer mailerWithDomainDefaults: dd]
+  messageForSent = nil;
+
+  // If we are encrypting mails, let's generate and
+  // send them individually
+  if ([self encrypt])
+    {
+      NSArray *recipients;
+      NSString *recipient;
+      int i;
+
+      recipients = [self allBareRecipients];
+
+      for (i = 0; i < [recipients count]; i++)
+        {
+          recipient = [recipients objectAtIndex: i];
+
+          if ([[context activeUser] hasEmail: recipient])
+            message = messageForSent = [self mimeMessageForRecipient: nil];
+          else
+            message = [self mimeMessageForRecipient: recipient];;
+
+          if (!message)
+            return  [NSException exceptionWithHTTPStatus: 500
+                                                  reason: @"could not generate message content"];
+
+          error = [[SOGoMailer mailerWithDomainDefaults: dd]
+                    sendMailData: message
+                    toRecipients: [NSArray arrayWithObject: recipient]
+                          sender: [self sender]
+                    withAuthenticator: [self authenticatorInContext: context]
+                       inContext: context];
+
+          if (error)
+            return error;
+        }
+
+      // If the current user isn't part of the recipient list for encrypted emails
+      // let's generate a crypted email for its sent folder.
+      if (!messageForSent)
+        messageForSent = [self mimeMessageForRecipient: nil];
+    }
+  else
+    {
+      // Encryption is done or not, if we didn't have to.
+      message = messageForSent = [self mimeMessageForRecipient: nil];
+
+      if (!message)
+        return  [NSException exceptionWithHTTPStatus: 500
+                                          reason: @"could not generate message content"];
+
+      error = [[SOGoMailer mailerWithDomainDefaults: dd]
                   sendMailData: message
                   toRecipients: [self allBareRecipients]
                         sender: [self sender]
-             withAuthenticator: [self authenticatorInContext: context]
+                withAuthenticator: [self authenticatorInContext: context]
                      inContext: context];
+    }
+
   if (!error && copyToSent)
     {
       sentFolder = [[self mailAccountFolder] sentFolderInContext: context];
@@ -1777,7 +2028,7 @@ static NSString    *userAgent      = nil;
         error = (NSException *) sentFolder;
       else
         {
-          error = [sentFolder postData: message flags: @"seen"];
+          error = [sentFolder postData: messageForSent flags: @"seen"];
           if (!error)
             {
               [self imap4Connection];
@@ -1828,7 +2079,7 @@ static NSString    *userAgent      = nil;
   NSString *str;
   NSData *message;
 
-  message = [self mimeMessageAsData];
+  message = [self mimeMessageForRecipient: nil];
   if (message)
     {
       str = [[NSString alloc] initWithData: message
