@@ -1,14 +1,15 @@
 /*
+  Copyright (C) 2005-2017 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
-  This file is part of OpenGroupware.org.
+  This file is part of SOGo.
 
-  OGo is free software; you can redistribute it and/or modify it under
+  SOGo is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
 
-  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
+  SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
   License for more details.
@@ -32,12 +33,16 @@
 #import <NGExtensions/NGBase64Coding.h>
 #import <NGExtensions/NGQuotedPrintableCoding.h>
 #import <NGImap4/NGImap4Connection.h>
+#import <NGMail/NGMimeMessage.h>
+#import <NGMime/NGMimeMultipartBody.h>
 
 #import <SoObjects/SOGo/NSDictionary+Utilities.h>
+#import <SoObjects/SOGo/NSObject+Utilities.h>
 
+#import "NSData+SMIME.h"
 #import "NSDictionary+Mail.h"
 #import "SOGoMailObject.h"
-
+#import "SOGoMailAccount.h"
 #import "SOGoMailBodyPart.h"
 
 @implementation SOGoMailBodyPart
@@ -165,35 +170,65 @@ static BOOL debugOn = NO;
 		    inContext: (WOContext *) localContext
 {
   // TODO: we might want to check for existence prior controller creation
-  Class clazz;
-  NSArray *subParts;
-  unsigned int nbr;
-  id obj;
   NSDictionary *subPart, *infos;
+  NSString *mimeType;
+  NSArray *subParts;
+  Class clazz;
+  id o, obj;
+
+  unsigned int nbr;
 
   nbr = [key intValue];
-  infos = [self partInfo];
-  subParts = [infos objectForKey: @"parts"];
-  if (!subParts)
-    subParts = [[infos objectForKey: @"body"] objectForKey: @"parts"];
+  o = [self container];
 
-  if (nbr > 0 && nbr < ([subParts count] + 1))
+  while (![o isKindOfClass: [SOGoMailObject class]])
+    o = [o container];
+
+  if ([o isEncrypted])
     {
-      subPart = [subParts objectAtIndex: nbr - 1];
-      clazz
-	= [[self class] bodyPartClassForMimeType:
-			  [subPart keysWithFormat: @"%{type}/%{subtype}"]
-			inContext: localContext];
-      obj = [clazz objectWithName: key inContainer: self];
+      NSData *certificate;
+      NGMimeMessage *m;
+      id part;
+
+      int i;
+
+      certificate = [[self mailAccountFolder] certificate];
+      m = [[o content] messageFromEncryptedDataAndCertificate: certificate];
+      part = [m body];
+
+      for (i = 0; i < [[self bodyPartPath] count]; i++)
+        {
+          nbr = [[[self bodyPartPath] objectAtIndex: i] intValue]-1;
+          part = [[part parts] objectAtIndex: nbr];;
+        }
+
+      //part = [[[m body] parts] objectAtIndex: ([key intValue]-1)];
+      part = [[part parts] objectAtIndex: ([key intValue]-1)];
+      mimeType = [[part contentType] stringValue];
+      clazz = [SOGoMailBodyPart bodyPartClassForMimeType: mimeType
+                                               inContext: localContext];
+      obj = [clazz objectWithName:key inContainer: self];
     }
   else
-    obj = self;
+    {
+      infos = [self partInfo];
+      subParts = [infos objectForKey: @"parts"];
+      if (!subParts)
+        subParts = [[infos objectForKey: @"body"] objectForKey: @"parts"];
 
-  return obj;  
-//   clazz = [SOGoMailBodyPart bodyPartClassForKey: _key
-// 			    inContext: _ctx];
+      if (nbr > 0 && nbr < ([subParts count] + 1))
+        {
+          subPart = [subParts objectAtIndex: nbr - 1];
+          mimeType = [subPart keysWithFormat: @"%{type}/%{subtype}"];
+          clazz = [[self class] bodyPartClassForMimeType: mimeType
+                                               inContext: localContext];
+          obj = [clazz objectWithName: key inContainer: self];
+        }
+      else
+        obj = self;
+    }
 
-//   return [clazz objectWithName: _key inContainer: self];
+  return obj;
 }
 
 - (NSString *) filename
@@ -250,10 +285,9 @@ static BOOL debugOn = NO;
   NSString *enc;
   NSData *data;
   
-  data = [[self imap4Connection] fetchContentOfBodyPart:
-				   [self bodyPartIdentifier]
-				 atURL:[self imap4URL]
-                                 withPeek: withPeek];
+  data = [[self imap4Connection] fetchContentOfBodyPart: [self bodyPartIdentifier]
+                                                  atURL: [self imap4URL]
+                                               withPeek: withPeek];
   if (data == nil) return nil;
 
   /* check for content encodings */
@@ -289,6 +323,41 @@ static BOOL debugOn = NO;
 
 - (NSData *) fetchBLOB
 {
+  id o;
+
+  // We check if the associated SOGoMailObject is encrypted, we must navigate
+  // in the container list until we find the proper container.
+  o = [self container];
+
+  while (![o isKindOfClass: [SOGoMailObject class]])
+    o = [o container];
+
+  if ([o isEncrypted])
+    {
+      NSData *certificate;
+      NGMimeMessage *m;
+      id part;
+
+      unsigned int i, nbr;
+
+      // No need to check if the cert is valid as we already do so
+      // in SOGoMailObject.
+      certificate = [[self mailAccountFolder] certificate];
+
+      m = [[o content] messageFromEncryptedDataAndCertificate: certificate];
+      part = [m body];
+
+      for (i = 0; i < [[self bodyPartPath] count]; i++)
+        {
+          nbr = [[[self bodyPartPath] objectAtIndex: i] intValue]-1;
+          part = [[part parts] objectAtIndex: nbr];;
+        }
+
+      return [part body];
+    }
+
+  // The mail is not encrypted, lets fetch the body party normally
+  // straight from the IMAP server
   return [self fetchBLOBWithPeek: NO];
 }
 
