@@ -49,6 +49,9 @@
 @interface SOGoMailCatalanForward : SOGoMailForward
 @end
 
+@interface SOGoMailChineseChinaForward : SOGoMailForward
+@end
+
 @interface SOGoMailChineseTaiwanForward : SOGoMailForward
 @end
 
@@ -88,6 +91,9 @@
 @interface SOGoMailItalianForward : SOGoMailForward
 @end
 
+@interface SOGoMailJapaneseForward : SOGoMailForward
+@end
+
 @interface SOGoMailLatvianForward : SOGoMailForward
 @end
 
