@@ -1,6 +1,6 @@
 /* SOGoMailForward.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2015 Inverse inc.
+ * Copyright (C) 2007-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -260,6 +260,9 @@
 @implementation SOGoMailCatalanForward
 @end
 
+@implementation SOGoMailChineseChinaForward
+@end
+
 @implementation SOGoMailChineseTaiwanForward
 @end
 
@@ -299,6 +302,9 @@
 @implementation SOGoMailItalianForward
 @end
 
+@implementation SOGoMailJapaneseForward
+@end
+
 @implementation SOGoMailLatvianForward
 @end
 
