@@ -48,6 +48,9 @@
 @interface SOGoMailCatalanReply : SOGoMailReply
 @end
 
+@interface SOGoMailChineseChinaReply : SOGoMailReply
+@end
+
 @interface SOGoMailChineseTaiwanReply : SOGoMailReply
 @end
 
@@ -87,6 +90,9 @@
 @interface SOGoMailItalianReply : SOGoMailReply
 @end
 
+@interface SOGoMailJapaneseReply : SOGoMailReply
+@end
+
 @interface SOGoMailLatvianReply : SOGoMailReply
 @end
 
