@@ -1,6 +1,6 @@
 /* SOGoMailReply.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2015 Inverse inc.
+ * Copyright (C) 2007-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -96,6 +96,9 @@
 @implementation SOGoMailCatalanReply
 @end
 
+@implementation SOGoMailChineseChinaReply
+@end
+
 @implementation SOGoMailChineseTaiwanReply
 @end
 
@@ -135,6 +138,9 @@
 @implementation SOGoMailItalianReply
 @end
 
+@implementation SOGoMailJapaneseReply
+@end
+
 @implementation SOGoMailLatvianReply
 @end
 
