@@ -21,30 +21,14 @@
 
 #import <unistd.h>
 
-#import <Foundation/NSBundle.h>
-#import <Foundation/NSClassDescription.h>
-#import <Foundation/NSFileManager.h>
-#import <Foundation/NSPathUtilities.h>
-#import <Foundation/NSSet.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSURL.h>
-#import <Foundation/NSValue.h>
 
 #import <NGObjWeb/SoClass.h>
 #import <NGObjWeb/SoObject+SoDAV.h>
-#import <NGObjWeb/WEClientCapabilities.h>
 #import <NGObjWeb/WOApplication.h>
-#import <NGObjWeb/WOContext.h>
-#import <NGObjWeb/WOResourceManager.h>
-#import <NGObjWeb/WOResponse.h>
-#import <NGObjWeb/WORequest.h>
 #import <NGObjWeb/WORequest+So.h>
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
-#import <DOM/DOMDocument.h>
-#import <DOM/DOMNode.h>
-#import <DOM/DOMProtocols.h>
 #import <NGCards/NSDictionary+NGCards.h>
 #import <SaxObjC/XMLNamespaces.h>
 
@@ -58,18 +42,12 @@
 #import "NSString+DAV.h"
 #import "NSString+Utilities.h"
 #import "SOGoCache.h"
-#import "SOGoDomainDefaults.h"
-#import "SOGoPermissions.h"
 #import "SOGoSystemDefaults.h"
 #import "SOGoUser.h"
-#import "SOGoUserDefaults.h"
-#import "SOGoUserFolder.h"
 #import "SOGoWebDAVAclManager.h"
-#import "SOGoWebDAVValue.h"
 #import "WORequest+SOGo.h"
 #import "WOResponse+SOGo.h"
 
-#import "SOGoObject.h"
 
 @implementation SOGoObject
 
@@ -1065,7 +1043,7 @@
   if (nameInContainer) 
     [_ms appendFormat:@" name=%@", nameInContainer];
   if (container)
-    [_ms appendFormat:@" container=0x%08X/%@", 
+    [_ms appendFormat:@" container=%p/%@", 
          container, [container valueForKey:@"nameInContainer"]];
 }
 
@@ -1074,7 +1052,7 @@
   NSMutableString *ms;
 
   ms = [NSMutableString stringWithCapacity:64];
-  [ms appendFormat:@"<0x%08X[%@]:", self, NSStringFromClass([self class])];
+  [ms appendFormat:@"<%p[%@]:", self, NSStringFromClass([self class])];
   [self appendAttributesToDescription:ms];
   [ms appendString:@">"];
 
@@ -1083,7 +1061,7 @@
 
 - (NSString *) loggingPrefix
 {
-  return [NSString stringWithFormat:@"<0x%08X[%@]:%@>",
+  return [NSString stringWithFormat:@"<%p[%@]:%@>",
 		   self, NSStringFromClass([self class]),
 		   [self nameInContainer]];
 }
@@ -1277,7 +1255,7 @@
       if (!cn)
 	cn = user;
       [userRecord appendFormat: @"<displayName>%@</displayName>",
-		  [cn stringByEscapingXMLString]];
+		  [cn safeStringByEscapingXMLString]];
     }
   
   if (![params containsObject: @"noemail"])
