@@ -32,10 +32,11 @@ class sogoToolTest(unittest.TestCase):
       self.assertEqual(os.getuid(), 0, "this test must run as root...")
 
       os.chown(self.backupdir, uid, gid)
-      status = os.system("su - %s -c \"(cd %s && %s backup . ALL >/dev/null 2>&1)\""
-                          % (sogo_user, self.backupdir, sogo_tool_path))
-
-      rc=os.WEXITSTATUS(status)
+      cmd = "sudo -u %s bash -c \"(cd %s && %s backup . ALL >/dev/null 2>&1)\"" % (sogo_user, self.backupdir, sogo_tool_path)
+      print "sogo-tool cmd to execute %s" % cmd
+      status = os.system(cmd)
+      print "Exit status of os.system(): %d" % status
+      rc = os.WEXITSTATUS(status)
       self.assertEqual(rc, 0, "sogo-tool failed RC=%d" % rc)
 
 
