@@ -199,14 +199,15 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
           folder = [sortedFolders objectAtIndex: i];
           //NSLog(@"  Address book: %@ (%@)", [folder displayName], [folder class]);
           contacts = [folder lookupContactsWithFilter: searchText
-                                           onCriteria: @"name_or_address"
+                                           onCriteria: nil
                                                sortBy: @"c_cn"
                                              ordering: NSOrderedAscending
                                              inDomain: domain];
           for (j = 0; j < [contacts count]; j++)
             {
               contact = [contacts objectAtIndex: j];
-              mail = [contact objectForKey: @"c_mail"];
+              [contact setObject: [folder displayName]
+                          forKey: @"containerName"];
               //NSLog(@"   found %@ (%@) ? %@", [contact objectForKey: @"c_name"], mail,
               //      [contact description]);
               if (!excludeLists && [[contact objectForKey: @"c_component"]
@@ -217,10 +218,17 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
                   [uniqueContacts setObject: contact 
                                      forKey: [contact objectForKey: @"c_name"]]; 
                 }
-              else if ([mail length]
-                       && [uniqueContacts objectForKey: mail] == nil
-                       && !(excludeGroups && [contact objectForKey: @"isGroup"]))
-                [uniqueContacts setObject: contact forKey: mail];
+              else
+                {
+                  if ([[contact objectForKey: @"emails"] count] == 0)
+                    // Contact must have an email address
+                    continue;
+                  mail = [[[contact objectForKey: @"emails"] objectAtIndex: 0] objectForKey: @"value"];
+                  if ([mail length]
+                      && [uniqueContacts objectForKey: mail] == nil
+                      && !(excludeGroups && [contact objectForKey: @"isGroup"]))
+                    [uniqueContacts setObject: contact forKey: mail];
+                }
             }
         }
       if ([uniqueContacts count] > 0)
@@ -343,6 +351,7 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
                                             && [currentFolder listRequiresDot]], @"listRequiresDot",
                                   acls, @"acls",
                                   urls, @"urls",
+                                  [currentFolder searchFields], @"searchFields",
                                   nil];
       [foldersAttrs addObject: folderAttrs];
     }
