@@ -1,6 +1,6 @@
 /* UIxMailPartSignedViewer.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2015 Inverse inc.
+ * Copyright (C) 2009-2018 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
  */
 
 #if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
-
 #include <openssl/bio.h>
 #include <openssl/err.h>
 #include <openssl/pkcs7.h>
@@ -29,6 +28,11 @@
 #import <Foundation/NSData.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSValue.h>
+
+#import <NGMime/NGMimeMultipartBody.h>
+
+#import <SOGo/NSString+Utilities.h>
+
 #import <Mailer/SOGoMailObject.h>
 
 #import "UIxMailRenderingContext.h"
@@ -51,19 +55,19 @@
 
   success = NO;
 
-  store = X509_STORE_new ();
-  OpenSSL_add_all_algorithms ();
+  store = X509_STORE_new();
+  OpenSSL_add_all_algorithms();
 
   if (store)
     {
-      lookup = X509_STORE_add_lookup (store, X509_LOOKUP_file());
+      lookup = X509_STORE_add_lookup(store, X509_LOOKUP_file());
       if (lookup)
         {
-          X509_LOOKUP_load_file (lookup, NULL, X509_FILETYPE_DEFAULT);
-          lookup = X509_STORE_add_lookup (store, X509_LOOKUP_hash_dir());
+          X509_LOOKUP_load_file(lookup, NULL, X509_FILETYPE_DEFAULT);
+          lookup = X509_STORE_add_lookup(store, X509_LOOKUP_hash_dir());
           if (lookup)
             {
-              X509_LOOKUP_add_dir (lookup, NULL, X509_FILETYPE_DEFAULT);
+              X509_LOOKUP_add_dir(lookup, NULL, X509_FILETYPE_DEFAULT);
               ERR_clear_error();
               success = YES;
             }
@@ -84,65 +88,71 @@
 
 - (void) _processMessage
 {
-  NSString *issuer, *subject;
   NSData *signedData;
   
   STACK_OF(X509) *certs;
   X509_STORE *x509Store;
   BIO *msgBio, *inData;
-  char sslError[1024];
+  const char* sslError;
   PKCS7 *p7;
   int err, i;
  
-  memset(sslError, 0, 1024);
-
   ERR_clear_error();
 
-  signedData = [[self clientObject] content];
+  if ([[self decodedFlatContent] isKindOfClass: [NGMimeMultipartBody class]])
+    signedData = [self flatContent];
+  else
+    signedData = [[self clientObject] content];
+
   msgBio = BIO_new_mem_buf ((void *) [signedData bytes], [signedData length]);
 
   inData = NULL;
   p7 = SMIME_read_PKCS7(msgBio, &inData);
 
-  subject = nil;
-  issuer = nil;
   certs = NULL;
+  certificates = [NSMutableArray array];
+  validationMessage = nil;
 
   if (p7)
     {
-      i = OBJ_obj2nid(p7->type);
-      
-      if (i == NID_pkcs7_signed)
+      if (OBJ_obj2nid(p7->type) == NID_pkcs7_signed)
 	{
+          NSString *subject, *issuer;
 	  X509 *x;
 	  
-	  certs=p7->d.sign->cert;
-	  
-	  if (sk_X509_num(certs) > 0)
-	    {
+	  certs = p7->d.sign->cert;
+
+          for (i = 0; i < sk_X509_num(certs); i++)
+            {
 	      BIO *buf;
-	      char p[256];
-	      
-	      memset(p, 0, 256);
-	      x = sk_X509_value(certs,0);
+	      char p[1024];
+
+	      x = sk_X509_value(certs, i);
+
+	      memset(p, 0, 1024);
 	      buf = BIO_new(BIO_s_mem());
-	      X509_NAME_print_ex(buf, X509_get_subject_name(x), 0, XN_FLAG_ONELINE & ~ASN1_STRFLGS_ESC_MSB);
-	      BIO_gets(buf, p, 256);
+	      X509_NAME_print_ex(buf, X509_get_subject_name(x), 0,
+                                 ASN1_STRFLGS_ESC_CTRL | XN_FLAG_SEP_MULTILINE | XN_FLAG_FN_LN);
+              BIO_read(buf, p, 1024);
 	      subject = [NSString stringWithUTF8String: p];
-	      
-	      memset(p, 0, 256);
-	      X509_NAME_print_ex(buf, X509_get_issuer_name(x), 0, XN_FLAG_ONELINE & ~ASN1_STRFLGS_ESC_MSB);
-	      BIO_gets(buf, p, 256);
-	      issuer = [NSString stringWithUTF8String: p];
-	      
 	      BIO_free(buf);
+
+	      memset(p, 0, 1024);
+	      buf = BIO_new(BIO_s_mem());
+	      X509_NAME_print_ex(buf, X509_get_issuer_name(x), 0,
+                                 ASN1_STRFLGS_ESC_CTRL | XN_FLAG_SEP_MULTILINE | XN_FLAG_FN_LN);
+	      BIO_read(buf, p, 1024);
+	      issuer = [NSString stringWithUTF8String: p];
+	      BIO_free(buf);
+
+              [certificates addObject: [self certificateForSubject: subject
+                                                         andIssuer: issuer]];
 	    }
 	}
       
       err = ERR_get_error();
       if (err)
 	{
-	  ERR_error_string_n (err, sslError, 1023);
 	  validSignature = NO;
 	}
       else
@@ -152,30 +162,29 @@
 					 NULL, PKCS7_DETACHED) == 1);
 	  
 	  err = ERR_get_error();
-	  if (err)
-	    ERR_error_string_n(err, sslError, 1023);
 	  
 	  if (x509Store)
 	    X509_STORE_free (x509Store);
 	}
+
+      if (err)
+        {
+          ERR_load_crypto_strings();
+          sslError = ERR_reason_error_string(err);
+          validationMessage = [[self labelForKey: [NSString stringWithUTF8String: sslError]] retain];
+        }
     }
+
   
   BIO_free (msgBio);
   if (inData)
     BIO_free (inData);
   
-  validationMessage = [NSMutableString string];
+  if (validSignature)
+    validationMessage = [NSString stringWithString: [self labelForKey: @"Message is signed"]];
+  else if (!validationMessage)
+    validationMessage = [NSString stringWithString: [self labelForKey: @"Digital signature is not valid"]];
 
-  if (!validSignature)
-    [validationMessage appendString: [self labelForKey: @"Digital signature is not valid"]];
-  else
-    [validationMessage appendString: [self labelForKey: @"Message is signed"]];
-  
-  if (issuer && subject)
-    [validationMessage appendFormat: @"\n%@: %@\n%@: %@",
-		   [self labelForKey: @"Subject"], subject,
-		     [self labelForKey: @"Issuer"], issuer];
-		   
   processed = YES;
 }
 
@@ -187,6 +196,20 @@
   return validSignature;
 }
 
+- (NSDictionary *) certificateForSubject: (NSString *) subject
+                               andIssuer: (NSString *) issuer
+{
+  return [NSDictionary dictionaryWithObjectsAndKeys:
+                              [subject componentsFromMultilineDN], @"subject",
+                              [issuer componentsFromMultilineDN], @"issuer",
+                       nil];
+}
+
+- (NSArray *) smimeCertificates
+{
+  return certificates;
+}
+
 - (NSString *) validationMessage
 {
   if (!processed)
@@ -200,6 +223,11 @@
   return NO;
 }
 
+- (NSArray *) smimeCertificates
+{
+  return nil;
+}
+
 - (BOOL) validSignature
 {
   return NO;
@@ -211,32 +239,53 @@
 }
 #endif
 
-- (id) renderedPart {
+- (id) renderedPart
+{
+  NSMutableArray *renderedParts;
   id info, viewer;
   NSArray *parts;
-  NSMutableArray *renderedParts;
+
   NSUInteger i, max;
 
-  parts = [[self bodyInfo] objectForKey: @"parts"];
+  if ([self decodedFlatContent])
+    parts = [[self decodedFlatContent] parts];
+  else
+    parts = [[self bodyInfo] objectForKey: @"parts"];
+
   max = [parts count];
   renderedParts = [NSMutableArray arrayWithCapacity: max];
+
   for (i = 0; i < max; i++)
     {
       [self setChildIndex: i];
-      [self setChildInfo: [parts objectAtIndex: i]];
+
+      if ([self decodedFlatContent])
+        [self setChildInfo: [[parts objectAtIndex: i] bodyInfo]];
+      else
+        [self setChildInfo: [parts objectAtIndex: i]];
+
       info = [self childInfo];
-      viewer = [[[self context] mailRenderingContext] viewerForBodyInfo:info];
+      viewer = [[[self context] mailRenderingContext] viewerForBodyInfo: info];
       [viewer setBodyInfo: info];
       [viewer setPartPath: [self childPartPath]];
+
+      if ([self decodedFlatContent])
+        [viewer setDecodedContent: [[parts objectAtIndex: i] body]];
+
+      [viewer setAttachmentIds: attachmentIds];
       [renderedParts addObject: [viewer renderedPart]];
     }
 
+  if (!processed)
+    [self _processMessage];
+
   return [NSDictionary dictionaryWithObjectsAndKeys:
                          [self className], @"type",
                        [NSNumber numberWithBool: [self supportsSMIME]], @"supports-smime",
                        [NSNumber numberWithBool: [self validSignature]], @"valid",
-                       [self validationMessage], @"error",
                        renderedParts, @"content",
+                       [self smimeCertificates], @"certificates",
+                       [self validationMessage], @"message",
                        nil];
 }
 
