@@ -1,6 +1,6 @@
 /* UIxMailAccountActions.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2016 Inverse inc.
+ * Copyright (C) 2007-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,34 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
+#include <openssl/bio.h>
+#include <openssl/err.h>
+#include <openssl/pem.h>
+#include <openssl/x509.h>
+#endif
+
+#import <Foundation/NSData.h>
 #import <Foundation/NSDictionary.h>
 
+#import <NGHttp/NGHttpRequest.h>
+
+#import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
+#define COMPILING_NGOBJWEB 1 /* httpRequest is needed in importCertificateAction */
 #import <NGObjWeb/WORequest.h>
+#undef COMPILING_NGOBJWEB
+#import <NGObjWeb/WOResponse.h>
+
+#import <NGMime/NGMimeBodyPart.h>
+#import <NGMime/NGMimeHeaderFields.h>
+#import <NGMime/NGMimeMultipartBody.h>
+#import <NGMime/NGMimeType.h>
+
 #import <NGImap4/NSString+Imap4.h>
 #import <NGExtensions/NSString+misc.h>
 
+#import <Mailer/NSData+SMIME.h>
 #import <Mailer/SOGoMailAccount.h>
 #import <Mailer/SOGoDraftObject.h>
 #import <Mailer/SOGoDraftsFolder.h>
@@ -91,7 +112,7 @@
   drafts = [[self clientObject] draftsFolderInContext: context];
   newDraftMessage = [drafts newDraft];
   headers = [NSMutableDictionary dictionary];
-  
+
   save = NO;
 
   value = [[self request] formValueForKey: @"mailto"];
@@ -177,4 +198,108 @@
   return [self _performDelegationAction: @selector (removeDelegates:)];
 }
 
+- (WOResponse *) certificateAction
+{
+  NSData *pem;
+  NSDictionary *data;
+  WOResponse *response;
+
+  pem = [[self clientObject] certificate];
+
+  if (pem)
+    {
+      data = [pem certificateDescription];
+      if (data)
+        {
+          response = [self responseWithStatus: 200  andJSONRepresentation: data];
+        }
+      else
+        {
+          data = [NSDictionary
+                   dictionaryWithObject: [self labelForKey: @"Error reading the certificate. Please install a new certificate."]
+                                 forKey: @"message"];
+          response = [self responseWithStatus: 500  andJSONRepresentation: data];
+        }
+    }
+  else
+    {
+      data = [NSDictionary
+               dictionaryWithObject: [self labelForKey: @"No certificate associated to account."]
+                             forKey: @"message"];
+      response = [self responseWithStatus: 404  andJSONRepresentation: data];
+    }
+
+  return response;
+}
+
+- (WOResponse *) importCertificateAction
+{
+  NSArray *parts;
+  NGMimeBodyPart *part;
+  NGMimeContentDispositionHeaderField *header;
+  NSData *pkcs12;
+  NSString *mimeType, *name, *password;
+  WOResponse *response;
+  id data;
+
+  unsigned int count, max;
+
+  password = nil;
+  pkcs12 = nil;
+  response = [self responseWithStatus: 507];
+
+  data = [[[context request] httpRequest] body];
+
+  if (![data isKindOfClass: [NSException class]])
+    {
+      parts = [data parts];
+      max = [parts count];
+      for (count = 0; count < max; count++)
+        {
+          part = [parts objectAtIndex: count];
+          header = (NGMimeContentDispositionHeaderField *)[part headerForKey: @"content-disposition"];
+          name = [header name];
+          if ([name isEqualToString: @"password"])
+            {
+              password = [part body];
+            }
+          else if ([name isEqualToString: @"file"])
+            {
+              mimeType = [(NGMimeType *)[part headerForKey: @"content-type"] stringValue];
+              if ([mimeType hasSuffix: @"pkcs12"])
+                {
+                  pkcs12 = [part body];
+                }
+              else
+                {
+                  response = [self responseWithStatus: 507];
+                }
+            }
+        }
+    }
+
+  if (password && pkcs12)
+    {
+      NSData *certificate;
+
+      certificate = [pkcs12 convertPKCS12ToPEMUsingPassword: password];
+
+      if (!certificate)
+        return [self responseWithStatus: 507];
+
+      [[self clientObject] setCertificate: certificate];
+
+      response = [self responseWith204];
+    }
+
+  return response;
+}
+
+- (WOResponse *) removeCertificateAction
+{
+  [[self clientObject] setCertificate: nil];
+
+  return [self responseWith204];
+}
+
 @end
