@@ -1,6 +1,6 @@
 /* UIxAppointmentEditor.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2016 Inverse inc.
+ * Copyright (C) 2007-2018 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #import <Foundation/NSValue.h>
 
 #import <NGObjWeb/SoSecurityManager.h>
+#import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGObjWeb/WORequest.h>
 #import <NGObjWeb/WOResponse.h>
 #import <NGObjWeb/NSException+HTTP.h>
@@ -45,6 +46,7 @@
 #import <Appointments/iCalCalendar+SOGo.h>
 #import <Appointments/iCalEntityObject+SOGo.h>
 #import <Appointments/iCalPerson+SOGo.h>
+#import <Appointments/iCalRepeatableEntityObject+SOGo.h>
 #import <Appointments/SOGoAppointmentFolder.h>
 #import <Appointments/SOGoAppointmentObject.h>
 #import <Appointments/SOGoAppointmentOccurence.h>
@@ -55,14 +57,14 @@
 
 @implementation UIxAppointmentEditor
 
-- (id) init
+- (id) initWithContext: (WOContext *) _context
 {
   SOGoUser *user;
 
-  if ((self = [super init]))
+  if ((self = [super initWithContext: _context]))
     {
-      user = [[self context] activeUser];
-      ASSIGN (dateFormatter, [user dateFormatterInContext: context]);
+      user = [_context activeUser];
+      ASSIGN (dateFormatter, [user dateFormatterInContext: _context]);
     }
 
   return self;
@@ -189,14 +191,20 @@
 
 - (NSException *) _adjustRecurrentRules
 {
-  iCalEvent *event;
-  iCalRecurrenceRule *rule;
+  NSArray *events;
+  NSCalendarDate *untilDate, *recurrenceId;
   NSEnumerator *rules;
   NSException *ex;
-  NSCalendarDate *untilDate;
-  SOGoUserDefaults *ud;
   NSTimeZone *timeZone;
+  SOGoAppointmentObject *co;
+  SOGoUserDefaults *ud;
+  iCalCalendar *calendar;
+  iCalEvent *event;
+  iCalRecurrenceRule *rule;
+  iCalRepeatableEntityObject *masterEvent, *occurrence;
+  int count, max;
 
+  co = [self clientObject];
   event = [self event];
   rules = [[event recurrenceRules] objectEnumerator];
   ex = nil;
@@ -234,6 +242,22 @@
         }
     }
 
+  // Remove invalid occurrences
+  calendar = [event parent];
+  events = [calendar events];
+  masterEvent = [events objectAtIndex: 0];
+  max = [events count];
+  for (count = max - 1; count > 0; count--)
+    {
+      occurrence = [events objectAtIndex: count];
+      recurrenceId = [occurrence recurrenceId];
+      if (recurrenceId && ![masterEvent doesOccurOnDate: recurrenceId])
+        {
+          [co prepareDeleteOccurence: (iCalEvent *)occurrence]; // notify attendees, update their calendars
+          [calendar removeChild: occurrence];
+        }
+    }
+
   return ex;
 }
 
@@ -650,6 +674,7 @@
  * @apiSuccess (Success 200) {Number} [repeat.days.occurence] Occurrence of a specific day within the monthly or yearly rule (values are -5 to 5)
  * @apiSuccess (Success 200) {Number[]} [repeat.months]       List of months of the year (values are 1 to 12)
  * @apiSuccess (Success 200) {Number[]} [repeat.monthdays]    Days of the month (values are 1 to 31)
+ * @apiSuccess (Success 200) {String[]} [repeat.dates]        Recurrence dates (ISO8601)
  */
 - (id <WOActionResults>) viewAction
 {
