@@ -139,8 +139,8 @@
   /**
    * @ngInject
    */
-  configure.$inject = ['$logProvider', '$compileProvider', '$httpProvider', '$mdThemingProvider', '$mdAriaProvider'];
-  function configure($logProvider, $compileProvider, $httpProvider, $mdThemingProvider, $mdAriaProvider) {
+  configure.$inject = ['$logProvider', '$compileProvider', '$httpProvider', '$mdThemingProvider', '$mdAriaProvider', '$qProvider'];
+  function configure($logProvider, $compileProvider, $httpProvider, $mdThemingProvider, $mdAriaProvider, $qProvider) {
     // Accent palette
     $mdThemingProvider.definePalette('sogo-green', {
       '50': 'eaf5e9',
@@ -202,6 +202,10 @@
       'contrastDefaultColor': 'dark',
       'contrastLightColors': ['800', '900']
     });
+    var greyMap = $mdThemingProvider.extendPalette('grey', {
+      '600': '00B0C0' // used when highlighting text in md-autocomplete
+    });
+    $mdThemingProvider.definePalette('sogo-grey', greyMap);
     // Default theme definition
     $mdThemingProvider.theme('default')
       .primaryPalette('sogo-blue', {
@@ -216,7 +220,8 @@
         'hue-1': 'A100', // grey-50
         'hue-2': '300',
         'hue-3': 'A700'
-      });
+      })
+      .backgroundPalette('sogo-grey');
 
     if (!DebugEnabled) {
       // Disable debugging information
@@ -224,6 +229,7 @@
       $compileProvider.debugInfoEnabled(false);
       // Disable warnings
       $mdAriaProvider.disableWarnings();
+      $qProvider.errorOnUnhandledRejections(false);
       // Disable theme generation but keep definition in config (required by mdColors)
       $mdThemingProvider.generateThemesOnDemand(true);
       // Disable theming completely
@@ -264,7 +270,7 @@
           // Handle CAS ticket renewal
           if ($window.usesCASAuthentication && rejection.status == -1) {
             deferred = $q.defer();
-            iframe = angular.element('<iframe class="ng-hide" src="' + UserFolderURL + 'recover"></iframe>');
+            iframe = angular.element('<iframe class="ng-hide" src="' + $window.UserFolderURL + 'recover"></iframe>');
             iframe.on('load', function() {
               // Once the browser has followed the redirection, send the initial request
               var $http = $injector.get('$http');
@@ -274,6 +280,9 @@
             document.body.appendChild(iframe[0]);
             return deferred.promise;
           }
+          else if ($window.usesSAML2Authentication && rejection.status == 401) {
+            $window.location.reload(true);
+          }
           else {
             // Broadcast the response error
             $rootScope.$broadcast('http:Error', rejection);
