@@ -43,7 +43,7 @@
       $Card: Card,
       $$Acl: Acl,
       $Preferences: Preferences,
-      $query: {search: 'name_or_address', value: '', sort: 'c_cn', asc: 1},
+      $query: {value: '', sort: 'c_cn', asc: 1},
       activeUser: Settings.activeUser(),
       selectedFolder: null,
       $refreshTimeout: null
@@ -474,7 +474,7 @@
 
     if (dry) {
       // Don't keep a copy of the query in dry mode
-      query = angular.copy(AddressBook.$query);
+      query = {value: '', sort: 'c_cn', asc: 1};
     }
     else {
       this.$isLoading = true;
@@ -497,7 +497,7 @@
       query.value = search;
 
     return _this.$id().then(function(addressbookId) {
-      var futureData = AddressBook.$$resource.fetch(addressbookId, 'view', query);
+      var futureData = AddressBook.$$resource.post(addressbookId, 'view', query);
 
       if (dry) {
         return futureData.then(function(response) {
@@ -512,15 +512,20 @@
             fields = _.invokeMap(response.headers[0], 'toLowerCase');
             idFieldIndex = fields.indexOf('id');
             response.headers.splice(0, 1);
+            results = _.map(response.headers, function(data) {
+              return data[idFieldIndex];
+            });
           }
 
-          if (excludedCards)
-            // Remove excluded cards from results
-            results = _.filter(response.ids, function(id) {
-              return _.isUndefined(_.find(excludedCards, _.bind(compareIds, id)));
-            });
-          else
-            results = response.ids;
+          if (response.ids) {
+            if (excludedCards)
+              // Remove excluded cards from results
+              results = _.filter(response.ids, function(id) {
+                return _.isUndefined(_.find(excludedCards, _.bind(compareIds, id)));
+              });
+            else
+              results = response.ids;
+          }
 
           // Remove cards that no longer match the search query
           for (index = cards.length - 1; index >= 0; index--) {
@@ -718,7 +723,12 @@
       data = { uids: _.map(selectedCards, 'id') };
     }
 
-    return AddressBook.$$resource.download(this.id, 'export', data, options);
+    if (data) {
+      return AddressBook.$$resource.download(this.id, 'export', data, options);
+    }
+    else {
+      return AddressBook.$$resource.open(this.id, 'export', data, options);
+    }
   };
 
   /**
