@@ -321,10 +321,10 @@
    * @function $filter
    * @memberof Mailbox.prototype
    * @desc Fetch the messages metadata of the mailbox
-   * @param {object} [sort] - sort preferences. Defaults to descendent by date.
-   * @param {string} sort.match - either AND or OR
-   * @param {string} sort.sort - either arrival, subject, from, to, date, or size
-   * @param {boolean} sort.asc - sort is ascendant if true
+   * @param {object} [sortsortingAttributes] - sort preferences. Defaults to descendent by date.
+   * @param {string} sortingAttributes.match - either AND or OR
+   * @param {string} sortingAttributes.sort - either arrival, subject, from, to, date, or size
+   * @param {boolean} sortingAttributes.asc - sort is ascendant if true
    * @param {object[]} [filters] - list of filters for the query
    * @param {string} filters.searchBy - either subject, from, to, cc, or body
    * @param {string} filters.searchInput - the search string to match
@@ -537,7 +537,6 @@
       // Move mailbox among its siblings according to its new name
       children.splice(i, 1);
       sibling = _.find(children, function(o) {
-        Mailbox.$log.debug(o.name + ' ? ' + _this.name);
         return (o.type == 'folder' && o.name.localeCompare(_this.name) > 0);
       });
       if (sibling) {
@@ -574,6 +573,7 @@
         // Update inbox quota
         if (data.quotas)
           _this.$account.updateQuota(data.quotas);
+        return true;
       });
   };
 
@@ -683,7 +683,7 @@
 
     options = { filename: this.name + '.zip' };
 
-    return Mailbox.$$resource.download(this.id, 'exportFolder', null, options);
+    return Mailbox.$$resource.open(this.id, 'exportFolder', null, options);
   };
 
   /**
@@ -893,6 +893,7 @@
 
     this.$futureMailboxData = futureMailboxData;
     this.$futureMailboxData.then(function(data) {
+      var selectedMessages = _.map(_this.$selectedMessages(), 'uid');
       Mailbox.$timeout(function() {
         var uids, headers;
 
@@ -916,16 +917,22 @@
 
           // Instanciate Message objects
           _.reduce(_this.uids, function(msgs, msg, i) {
-            var data;
-            if (_this.threaded)
+            var data, msgObject;
+            if (_this.threaded) {
               data = _.zipObject(uids, msg);
-            else
+            } else {
               data = {uid: msg.toString()};
+            }
 
             // Build map of UID <=> index
             _this.uidsMap[data.uid] = i;
 
-            msgs.push(new Mailbox.$Message(_this.$account.id, _this, data, true));
+            msgObject = new Mailbox.$Message(_this.$account.id, _this, data, true);
+
+            // Restore selection
+            msgObject.selected = selectedMessages.indexOf(msgObject.uid) > -1;
+
+            msgs.push(msgObject);
 
             return msgs;
           }, _this.$messages);
@@ -934,7 +941,7 @@
           _.forEach(_this.headers, function(data) {
             var msg = _.zipObject(headers, data),
                 i = _this.uidsMap[msg.uid.toString()];
-            _.extend(_this.$messages[i], msg);
+            _this.$messages[i].init(msg);
           });
         }
         Mailbox.$log.debug('mailbox ' + _this.id + ' ready');
@@ -971,7 +978,7 @@
             messageHeaders = _.zipObject(headers, messageHeaders);
             j = _this.uidsMap[messageHeaders.uid.toString()];
             if (angular.isDefined(j)) {
-              _.extend(_this.$messages[j], messageHeaders);
+              _this.$messages[j].init(messageHeaders);
             }
           });
         }
