@@ -6,11 +6,20 @@
   /**
    * @ngInject
    */
-  MailboxController.$inject = ['$window', '$scope', '$timeout', '$q', '$state', '$mdDialog', '$mdToast', 'stateAccounts', 'stateAccount', 'stateMailbox', 'sgHotkeys', 'encodeUriFilter', 'sgSettings', 'sgFocus', 'Dialog', 'Account', 'Mailbox'];
-  function MailboxController($window, $scope, $timeout, $q, $state, $mdDialog, $mdToast, stateAccounts, stateAccount, stateMailbox, sgHotkeys, encodeUriFilter, sgSettings, focus, Dialog, Account, Mailbox) {
+  MailboxController.$inject = ['$window', '$scope', '$timeout', '$q', '$state', '$mdDialog', '$mdToast', 'stateAccounts', 'stateAccount', 'stateMailbox', 'sgHotkeys', 'encodeUriFilter', 'sgSettings', 'sgFocus', 'Dialog', 'Preferences', 'Account', 'Mailbox'];
+  function MailboxController($window, $scope, $timeout, $q, $state, $mdDialog, $mdToast, stateAccounts, stateAccount, stateMailbox, sgHotkeys, encodeUriFilter, sgSettings, focus, Dialog, Preferences, Account, Mailbox) {
     var vm = this,
         defaultWindowTitle = angular.element($window.document).find('title').attr('sg-default') || "SOGo",
-        hotkeys = [];
+        hotkeys = [],
+        sortLabels;
+
+    sortLabels = {
+      subject: 'Subject',
+      from: 'From',
+      date: 'Date',
+      size: 'Size',
+      arrival: 'Order Received'
+    };
 
     this.$onInit = function() {
       // Expose controller for eventual popup windows
@@ -117,14 +126,28 @@
       return vm.selectedFolder.$compact();
     }
 
+    this.centerIsClose = function(navController_centerIsClose) {
+      // Allow the messages list to be hidden only if a message is selected
+      return this.selectedFolder.hasSelectedMessage() && !!navController_centerIsClose;
+    };
+
     this.sort = function(field) {
-      vm.selectedFolder.$filter({ sort: field });
+      if (field) {
+        vm.selectedFolder.$filter({ sort: field });
+      }
+      else {
+        return sortLabels[vm.service.$query.sort];
+      }
     };
 
     this.sortedBy = function(field) {
       return Mailbox.$query.sort == field;
     };
 
+    this.ascending = function() {
+      return Mailbox.$query.asc;
+    };
+
     this.searchMode = function() {
       vm.mode.search = true;
       focus('search');
@@ -141,11 +164,15 @@
       });
     };
 
+    this.composeWindowEnabled = function() {
+      return Preferences.defaults.SOGoMailComposeWindowEnabled;
+    };
+
     this.newMessage = function($event, inPopup) {
       var message;
 
       if (vm.messageDialog === null) {
-        if (inPopup)
+        if (inPopup || Preferences.defaults.SOGoMailComposeWindow == 'popup')
           _newMessageInPopup();
         else {
           message = vm.account.$newMessage();
@@ -163,6 +190,7 @@
                 stateMessage: message
               }
             })
+            .catch(_.noop) // Cancel
             .finally(function() {
               vm.messageDialog = null;
             });
@@ -212,7 +240,7 @@
       }
 
       if (index > -1)
-        vm.selectMessage(vm.selectedFolder.$messages[index]);
+        vm.selectMessage(vm.selectedFolder.getItemAtIndex(index));
 
       $event.preventDefault();
 
@@ -235,7 +263,7 @@
         index = 0;
 
       if (index < vm.selectedFolder.getLength())
-        vm.selectMessage(vm.selectedFolder.$messages[index]);
+        vm.selectMessage(vm.selectedFolder.getItemAtIndex(index));
       else
         index = -1;
 
