@@ -23,8 +23,7 @@
     if (typeof futureMessageData.then !== 'function') {
       //console.debug(JSON.stringify(futureMessageData, undefined, 2));
       if (angular.isUndefined(lazy) || !lazy) {
-        angular.extend(this, futureMessageData);
-        this.$formatFullAddresses();
+        this.init(futureMessageData);
       }
       this.uid = parseInt(futureMessageData.uid);
     }
@@ -46,6 +45,7 @@
       $timeout: $timeout,
       $log: $log,
       $$resource: new Resource(Settings.activeUser('folderURL') + 'Mail', Settings.activeUser()),
+      $Preferences: Preferences,
       $avatar: angular.bind(Preferences, Preferences.avatar)
     });
 
@@ -103,6 +103,24 @@
     return results;
   };
 
+  /**
+   * @function init
+   * @memberof Message.prototype
+   * @desc Extend instance with new data and massage some attributes.
+   * @param {object} data - attributes of message
+   */
+  Message.prototype.init = function(data) {
+    var _this = this;
+    angular.extend(this, data);
+    this.$formatFullAddresses();
+    this.$loadUnsafeContent = false;
+    _.forEach(this.flags, function(flag, i) {
+      if (flag.charAt(0) == '$') {
+        _this.flags.splice(i, 1,'_' + flag);
+      }
+    });
+  };
+
   /**
    * @function $absolutePath
    * @memberof Message.prototype
@@ -273,6 +291,7 @@
    */
   Message.prototype.loadUnsafeContent = function() {
     this.$loadUnsafeContent = true;
+    delete this.$parts;
   };
 
   /**
@@ -294,14 +313,21 @@
           // Can be used for UIxMailPartMixedViewer, UIxMailPartMessageViewer, and UIxMailPartSignedViewer
           else if (angular.isArray(part.content)) {
             if (part.type == 'UIxMailPartSignedViewer' && part['supports-smime'] === 1) {
-              // First line in a h1, others each in a p
-              var formattedMessage = "<p>" + part.error.replace(/\n/, "</p><p class=\"md-caption\">");
-              formattedMessage = formattedMessage.replace(/\n/g, "</p><p class=\"md-caption\">") + "</p>";
-              _this.$smime = {
-                validSignature: part.valid,
-                message: formattedMessage
+              _this.signed = {
+                valid: part.valid,
+                certificate: part.certificates[part.certificates.length - 1],
+                message: part.message
               };
             }
+            else if (part.type == 'UIxMailPartEncryptedViewer') {
+              _this.encrypted = {
+                valid: part.valid
+              };
+              if (part.valid)
+                _this.encrypted.message = l("This message is encrypted");
+              else
+                _this.encrypted.message = l("This message can't be decrypted. Please make sure you have uploaded your S/MIME certificate from the mail preferences module.");
+            }
             _.forEach(part.content, function(mixedPart) {
               _visit(mixedPart);
             });
@@ -360,9 +386,16 @@
           }
         };
 
-    if (this.parts)
+    if (this.$parts)
+      // Use the cache
+      return this.$parts;
+
+    else if (this.parts)
       _visit(this.parts);
 
+    // Cache result
+    this.$parts = parts;
+
     return parts;
   };
 
@@ -385,6 +418,13 @@
         });
         if (identity)
           data.from = identity.full;
+        var accountDefaults = Message.$Preferences.defaults.AuxiliaryMailAccounts[_this.$mailbox.$account.id];
+        if (accountDefaults.security) {
+          if (accountDefaults.security.alwaysSign)
+            data.sign = true;
+          if (accountDefaults.security.alwaysEncrypt)
+            data.encrypt = true;
+        }
         Message.$log.debug('editable = ' + JSON.stringify(data, undefined, 2));
         angular.extend(_this.editable, data);
         return data.text;
@@ -435,7 +475,7 @@
     var data = {
       operation: operation,
       msgUIDs: [this.uid],
-      flags: tag
+      flags: tag.replace(/^_\$/, '$')
     };
 
     if (tag)
@@ -544,6 +584,41 @@
     return this.$unwrap(futureMessageData);
   };
 
+  /**
+   * @function $parseMailto
+   * @memberof Message.prototype
+   * @desc Extend the editable content of the message with the
+   * information parsed from the specified "mailto:" link.
+   */
+  Message.prototype.$parseMailto = function(mailto) {
+    var to, data, match = /^mailto:([^\?]+)/.exec(mailto);
+    if (match) {
+      // Recipients
+      to = _.map(decodeURIComponent(match[1]).split(','), function(email) {
+        return '<' + email.trim() + '>';
+      });
+      data = { to: to };
+      // Subject & body
+      _.forEach(['subject', 'body'], function(param) {
+        var re = new RegExp(param + '=([^&]+)');
+        param = (param == 'body')? 'text' : param;
+        match = re.exec(mailto);
+        if (match)
+          data[param] = decodeURIComponent(match[1]);
+      });
+      // Other Recipients
+      _.forEach(['cc', 'bcc'], function(param) {
+        var re = new RegExp(param + '=([^&]+)');
+        match = re.exec(mailto);
+        if (match)
+          data[param] = _.map(decodeURIComponent(match[1]).split(','), function(email) {
+            return '<' + email.trim() + '>';
+          });
+      });
+      angular.extend(this.editable, data);
+    }
+  };
+
   /**
    * @function $reply
    * @memberof Message.prototype
@@ -599,6 +674,13 @@
       // Fetch draft initial data
       return Message.$$resource.fetch(message.$absolutePath({asDraft: true}), 'edit').then(function(data) {
         Message.$log.debug('New ' + action + ': ' + JSON.stringify(data, undefined, 2) + ' original UID: ' + _this.uid);
+        var accountDefaults = Message.$Preferences.defaults.AuxiliaryMailAccounts[_this.$mailbox.$account.id];
+        if (accountDefaults.security) {
+          if (accountDefaults.security.alwaysSign)
+            data.sign = true;
+          if (accountDefaults.security.alwaysEncrypt)
+            data.encrypt = true;
+        }
         angular.extend(message.editable, data);
 
         // We keep a reference to our original message in order to update the flags
@@ -680,10 +762,9 @@
         _this.$mailbox.unseenCount--;
       }
       return Message.$timeout(function() {
-        angular.extend(_this, data);
-        _this.$formatFullAddresses();
-        _this.$loadUnsafeContent = false;
+        delete _this.$parts;
         _this.$loaded = Message.STATUS.LOADED;
+        _this.init(data);
         return _this;
       });
     });
