@@ -6,8 +6,8 @@
   /**
    * @ngInject
    */
-  MessageController.$inject = ['$window', '$scope', '$state', '$mdMedia', '$mdDialog', 'sgConstant', 'stateAccounts', 'stateAccount', 'stateMailbox', 'stateMessage', 'sgHotkeys', 'encodeUriFilter', 'sgSettings', 'ImageGallery', 'sgFocus', 'Dialog', 'Calendar', 'Component', 'Account', 'Mailbox', 'Message'];
-  function MessageController($window, $scope, $state, $mdMedia, $mdDialog, sgConstant, stateAccounts, stateAccount, stateMailbox, stateMessage, sgHotkeys, encodeUriFilter, sgSettings, ImageGallery, focus, Dialog, Calendar, Component, Account, Mailbox, Message) {
+  MessageController.$inject = ['$window', '$scope', '$state', '$mdMedia', '$mdDialog', 'sgConstant', 'stateAccounts', 'stateAccount', 'stateMailbox', 'stateMessage', 'sgHotkeys', 'encodeUriFilter', 'sgSettings', 'ImageGallery', 'sgFocus', 'Dialog', 'Preferences', 'Calendar', 'Component', 'Account', 'Mailbox', 'Message'];
+  function MessageController($window, $scope, $state, $mdMedia, $mdDialog, sgConstant, stateAccounts, stateAccount, stateMailbox, stateMessage, sgHotkeys, encodeUriFilter, sgSettings, ImageGallery, focus, Dialog, Preferences, Calendar, Component, Account, Mailbox, Message) {
     var vm = this, popupWindow = null, hotkeys = [];
 
     this.$onInit = function() {
@@ -26,13 +26,13 @@
       this.tags = { searchText: '', selected: '' };
       this.showFlags = stateMessage.flags && stateMessage.flags.length > 0;
       this.$showDetailedRecipients = false;
-
-      vm.showRawSource = false;
+      this.showRawSource = false;
 
       _registerHotkeys(hotkeys);
 
       // One-way refresh of the parent window when modifying the message from a popup window.
-      if ($window.opener) {
+      if ($window.opener &&
+          '$mailboxController' in $window.opener) {
         // Update the message flags. The message must be displayed in the parent window.
         $scope.$watchCollection(function() { return vm.message.flags; }, function(newTags, oldTags) {
           var ctrls;
@@ -160,12 +160,13 @@
       var message, mailbox, ctrls = {};
       if ($window.opener) {
         // Deleting the message from a popup window
-        if ($window.opener.$mailboxController &&
+        if ('$mailboxController' in $window.opener &&
+            'selectedFolder' in $window.opener.$mailboxController &&
             $window.opener.$mailboxController.selectedFolder.$id() == stateMailbox.$id()) {
             // The message mailbox is opened in the parent window
             mailbox = $window.opener.$mailboxController;
             ctrls.mailboxCtrl = mailbox;
-            if ($window.opener.$messageController &&
+            if ('$messageController' in $window.opener &&
                 $window.opener.$messageController.message.uid == stateMessage.uid) {
               // The message is opened in the parent window
               message = $window.opener.$messageController;
@@ -177,6 +178,8 @@
     }
 
     this.addFlags = function($event) {
+      $event.stopPropagation();
+      $event.preventDefault();
       this.showFlags = true;
       focus("flags");
     };
@@ -193,27 +196,8 @@
         href = $event.target.attributes.href.value;
         match = /^mailto:([^\?]+)/.exec(href);
         if (match) {
-          // Recipients
-          to = _.map(decodeURIComponent(match[1]).split(','), function(email) {
-            return '<' + email + '>';
-          });
-          data = { to: to };
-          // Subject & body
-          _.forEach(['subject', 'body'], function(param) {
-            var re = new RegExp(param + '=([^&]+)');
-            param = (param == 'body')? 'text' : param;
-            match = re.exec(href);
-            if (match)
-              data[param] = [decodeURIComponent(match[1])];
-          });
-          // Recipients
-          _.forEach(['cc', 'bcc'], function(param) {
-            var re = new RegExp(param + '=([^&]+)');
-            match = re.exec(href);
-            if (match)
-              data[param] = [decodeURIComponent(match[1])];
-          });
-          this.newMessage($event, data); // will stop event propagation
+          delete $event.target.attributes.target;
+          this.newMessage($event, href); // will stop event propagation
         }
       }
     };
@@ -232,6 +216,9 @@
         message = stateMessage;
         state = $state;
       }
+      if (Mailbox.$virtualMode) {
+        mailbox = Mailbox.selectedFolder; // the VirtualMailbox instance
+      }
 
       mailbox.$deleteMessages([message]).then(function(index) {
         var nextIndex = index;
@@ -241,10 +228,10 @@
           // Select either the next or previous message
           if (index > 0) {
             nextIndex -= 1;
-            nextMessage = mailbox.$messages[nextIndex];
+            nextMessage = mailbox.getItemAtIndex(nextIndex);
           }
-          if (index < mailbox.$messages.length)
-            previousMessage = mailbox.$messages[index];
+          if (index < mailbox.getLength())
+            previousMessage = mailbox.getItemAtIndex(index);
 
           if (nextMessage) {
             if (nextMessage.isread && previousMessage && !previousMessage.isread) {
@@ -259,7 +246,10 @@
 
           try {
             if (nextMessage && $mdMedia(sgConstant['gt-md'])) {
-              state.go('mail.account.mailbox.message', { messageId: nextMessage.uid });
+              if (Mailbox.$virtualMode)
+                state.go('mail.account.virtualMailbox.message', {mailboxId: encodeUriFilter(nextMessage.$mailbox.path), messageId: nextMessage.uid});
+              else
+                state.go('mail.account.mailbox.message', {messageId: nextMessage.uid});
               if (nextIndex < mailbox.$topIndex)
                 mailbox.$topIndex = nextIndex;
               else if (nextIndex > mailbox.$lastVisibleIndex)
@@ -295,6 +285,7 @@
                 stateMessage: message
               }
             })
+            .catch(_.noop) // Cancel
             .finally(function() {
               _messageDialog(null);
               vm.closePopup();
@@ -303,35 +294,50 @@
       }
     }
 
+    this._showMailEditorInPopup = function(action) {
+      if (!sgSettings.isPopup &&
+          Preferences.defaults.SOGoMailComposeWindow == 'popup') {
+        this.openInPopup(action);
+        return true;
+      }
+      return false;
+    };
+
     this.close = function() {
-      $state.go('mail.account.mailbox').then(function() {
+      var destination = Mailbox.$virtualMode ? 'mail.account.virtualMailbox' : 'mail.account.mailbox';
+      $state.go(destination).then(function() {
         vm.message = null;
         delete stateMailbox.selectedMessage;
       });
     };
 
     this.reply = function($event) {
-      var message = this.message.$reply();
-      _showMailEditor($event, message);
+      if (!this._showMailEditorInPopup('reply')) {
+        _showMailEditor($event, this.message.$reply());
+      }
     };
 
     this.replyAll = function($event) {
-      var message = this.message.$replyAll();
-      _showMailEditor($event, message);
+      if (!this._showMailEditorInPopup('replyall')) {
+        _showMailEditor($event, this.message.$replyAll());
+      }
     };
 
     this.forward = function($event) {
-      var message = this.message.$forward();
-      _showMailEditor($event, message);
+      if (!this._showMailEditorInPopup('forward')) {
+        _showMailEditor($event, this.message.$forward());
+      }
     };
 
     this.edit = function($event) {
-      this.message.$editableContent().then(function() {
-        _showMailEditor($event, vm.message);
-      });
+      if (!this._showMailEditorInPopup('edit')) {
+        this.message.$editableContent().then(function() {
+          _showMailEditor($event, vm.message);
+        });
+      }
     };
 
-    this.openPopup = function() {
+    this.openInPopup = function(action) {
       var url = [sgSettings.baseURL(),
                  'UIxMailPopupView#!/Mail',
                  this.message.accountId,
@@ -340,6 +346,7 @@
                  this.message.uid]
           .join('/'),
           wId = this.message.$absolutePath();
+      if (action) url += '/' + action;
       popupWindow = $window.open(url, wId,
                                  ["width=680",
                                   "height=520",
@@ -355,17 +362,16 @@
     };
 
     this.closePopup = function() {
-      if ($window.opener)
+      if ($window.document.body.classList.contains('popup'))
         $window.close();
     };
 
-    this.newMessage = function($event, editableContent) {
-      this.account.$newMessage().then(function(message) {
-        angular.extend(message.editable, editableContent);
-        _showMailEditor($event, message);
-      });
+    this.newMessage = function($event, mailto) {
       $event.stopPropagation();
       $event.preventDefault();
+      this.account.$newMessage({ mailto: mailto }).then(function(message) {
+        _showMailEditor($event, message);
+      });
     };
 
     this.toggleRawSource = function($event) {
