@@ -52,14 +52,13 @@
   /**
    * @ngInject
    */
-  sgMailboxListItemController.$inject = ['$scope', '$element', '$state', '$mdToast', '$mdPanel', '$mdMedia', '$mdSidenav', 'sgConstant', 'Dialog', 'Mailbox', 'encodeUriFilter'];
-  function sgMailboxListItemController($scope, $element, $state, $mdToast, $mdPanel, $mdMedia, $mdSidenav, sgConstant, Dialog, Mailbox, encodeUriFilter) {
+  sgMailboxListItemController.$inject = ['$scope', '$element', '$state', '$timeout', '$mdToast', '$mdPanel', '$mdMedia', '$mdSidenav', 'sgConstant', 'Dialog', 'Mailbox', 'encodeUriFilter'];
+  function sgMailboxListItemController($scope, $element, $state, $timeout, $mdToast, $mdPanel, $mdMedia, $mdSidenav, sgConstant, Dialog, Mailbox, encodeUriFilter) {
     var $ctrl = this;
 
 
     this.$onInit = function() {
       this.$element = $element;
-      this.service = Mailbox;
       this.editMode = false;
       this.accountController.addMailboxController(this);
     };
@@ -84,7 +83,7 @@
 
 
     this.selectFolder = function($event) {
-      if (this.editMode || this.mailbox == Mailbox.selectedFolder)
+      if (this.editMode || this.mailbox == Mailbox.selectedFolder || this.mailbox.isNoSelect())
         return;
       Mailbox.$virtualPath = false;
       Mailbox.$virtualMode = false;
@@ -106,15 +105,24 @@
 
 
     this.editFolder = function($event) {
+      $event.stopPropagation();
+      $event.preventDefault();
       this.editMode = true;
       this.inputElement.value = this.mailbox.name;
       this.clickableElement.classList.add('ng-hide');
       this.inputContainer.classList.remove('ng-hide');
-      this.inputElement.focus();
-      this.inputElement.select();
-      if ($event) {
-        $event.stopPropagation();
-        $event.preventDefault();
+      if ($event.srcEvent.type == 'touchend') {
+        $timeout(function() {
+          $ctrl.inputElement.select();
+          $ctrl.inputElement.focus();
+        }, 200); // delayed focus for iOS
+      }
+      else {
+        this.inputElement.select();
+        this.inputElement.focus();
+      }
+      if (this.panel) {
+        this.panel.close();
       }
     };
 
@@ -189,7 +197,8 @@
         locals: {
           itemCtrl: this,
           folder: this.mailbox,
-          confirmDelete: this.confirmDelete
+          editFolder: angular.bind(this, this.editFolder),
+          confirmDelete: angular.bind(this, this.confirmDelete)
         },
         bindToController: true,
         controller: MenuController,
@@ -206,6 +215,7 @@
 
       $mdPanel.open(config)
         .then(function(panelRef) {
+          $ctrl.panel = panelRef;
           // Automatically close panel when clicking inside of it
           panelRef.panelEl.one('click', function() {
             panelRef.close();
@@ -234,10 +244,6 @@
             });
         };
 
-        this.editFolder = function() {
-          this.itemCtrl.editFolder();
-        };
-
         this.compactFolder = function() {
           this.folder.$compact().then(function() {
             $mdToast.show(
