@@ -9,7 +9,7 @@
    */
   PreferencesController.$inject = ['$q', '$window', '$state', '$mdMedia', '$mdSidenav', '$mdDialog', '$mdToast', 'sgSettings', 'sgFocus', 'Dialog', 'User', 'Account', 'Preferences', 'Authentication'];
   function PreferencesController($q, $window, $state, $mdMedia, $mdSidenav, $mdDialog, $mdToast, sgSettings, focus, Dialog, User, Account, Preferences, Authentication) {
-    var vm = this, account, mailboxes = [], today = new Date(), tomorrow = today.beginOfDay().addDays(1);
+    var vm = this, account, mailboxes = [], today = new Date().beginOfDay();
 
     this.$onInit = function() {
       this.preferences = Preferences;
@@ -17,6 +17,7 @@
       this.timeZonesList = $window.timeZonesList;
       this.timeZonesSearchText = '';
       this.sieveVariablesCapability = ($window.sieveCapabilities.indexOf('variables') >= 0);
+      this.mailLabelKeyRE = new RegExp(/^(?!^_\$)[^(){} %*\"\\\\]*?$/);
 
 
       if (sgSettings.activeUser('path').mail) {
@@ -43,7 +44,7 @@
     this.go = function(module, form) {
       if (form.$valid) {
         // Close sidenav on small devices
-        if ($mdMedia('xs'))
+        if (!$mdMedia('gt-sm'))
           $mdSidenav('left').close();
         $state.go('preferences.' + module);
       }
@@ -62,8 +63,8 @@
     };
 
     this.addCalendarCategory = function(form) {
-      this.preferences.defaults.SOGoCalendarCategoriesColors["New category"] = "#aaa";
-      this.preferences.defaults.SOGoCalendarCategories.push("New category");
+      this.preferences.defaults.SOGoCalendarCategoriesColors[l('New category')] = "#aaa";
+      this.preferences.defaults.SOGoCalendarCategories.push(l('New category'));
       focus('calendarCategory_' + (this.preferences.defaults.SOGoCalendarCategories.length - 1));
       form.$setDirty();
     };
@@ -76,8 +77,12 @@
     };
 
     this.addContactCategory = function(form) {
-      this.preferences.defaults.SOGoContactsCategories.push("");
-      focus('contactCategory_' + (this.preferences.defaults.SOGoContactsCategories.length - 1));
+      var i = _.indexOf(this.preferences.defaults.SOGoContactsCategories, "");
+      if (i < 0) {
+        this.preferences.defaults.SOGoContactsCategories.push("");
+        i = this.preferences.defaults.SOGoContactsCategories.length - 1;
+      }
+      focus('contactCategory_' + i);
       form.$setDirty();
     };
 
@@ -94,6 +99,7 @@
       account = _.last(this.preferences.defaults.AuxiliaryMailAccounts);
       angular.extend(account,
                      {
+                       isNew: true,
                        name: "",
                        identities: [
                          {
@@ -143,6 +149,8 @@
       }).then(function() {
         vm.preferences.defaults.AuxiliaryMailAccounts[index] = account;
         form.$setDirty();
+      }, function() {
+        // Cancel
       });
     };
 
@@ -150,22 +158,28 @@
       this.preferences.defaults.AuxiliaryMailAccounts.splice(index, 1);
       form.$setDirty();
     };
-    
+
+    this.resetMailLabelValidity = function(index, form) {
+      form['mailIMAPLabel_' + index].$setValidity('duplicate', true);
+    };
+
     this.addMailLabel = function(form) {
       // See $omit() in the Preferences services for real key generation
       var key = '_$$' + guid();
-      this.preferences.defaults.SOGoMailLabelsColors[key] =  ["New label", "#aaa"];
-      focus('mailLabel_' + (_.size(this.preferences.defaults.SOGoMailLabelsColors) - 1));
+      this.preferences.defaults.SOGoMailLabelsColorsKeys.push("label");
+      this.preferences.defaults.SOGoMailLabelsColorsValues.push(["New label", "#aaa"]);
+      focus('mailLabel_' + (_.size(this.preferences.defaults.SOGoMailLabelsColorsKeys) - 1));
       form.$setDirty();
     };
 
-    this.removeMailLabel = function(key, form) {
-      delete this.preferences.defaults.SOGoMailLabelsColors[key];
+    this.removeMailLabel = function(index, form) {
+      this.preferences.defaults.SOGoMailLabelsColorsKeys.splice(index, 1);
+      this.preferences.defaults.SOGoMailLabelsColorsValues.splice(index, 1);
       form.$setDirty();
     };
 
     this.addMailFilter = function(ev, form) {
-      var filter = { match: 'all' };
+      var filter = { match: 'all', active: 1 };
 
       $mdDialog.show({
         templateUrl: 'editFilter?filter=new',
@@ -243,7 +257,7 @@
     this.confirmChanges = function($event, form) {
       var target;
 
-      if (form.$dirty && $form.$valid) {
+      if (form.$dirty && form.$valid) {
         // Stop default action
         $event.preventDefault();
         $event.stopPropagation();
@@ -306,6 +320,21 @@
         }
       }
 
+      if (this.preferences.defaults.SOGoMailLabelsColorsKeys.length !=
+          this.preferences.defaults.SOGoMailLabelsColorsValues.length ||
+          this.preferences.defaults.SOGoMailLabelsColorsKeys.length !=
+          _.uniq(this.preferences.defaults.SOGoMailLabelsColorsKeys).length) {
+        Dialog.alert(l('Error'), l("IMAP labels must have unique names."));
+        _.forEach(this.preferences.defaults.SOGoMailLabelsColorsKeys, function (value, i, keys) {
+          if (form['mailIMAPLabel_' + i].$dirty &&
+              (keys.indexOf(value) != i ||
+               keys.indexOf(value, i+1) > -1)) {
+            form['mailIMAPLabel_' + i].$setValidity('duplicate', false);
+            sendForm = false;
+          }
+        });
+      }
+
       if (sendForm)
         return this.preferences.$save().then(function(data) {
           if (!options || !options.quick) {
@@ -382,9 +411,6 @@
           v.startDate = new Date(v.endDate.getTime());
           v.startDate.addDays(-1);
         }
-        if (v.startDate.getTime() < tomorrow.getTime()) {
-          v.startDate = new Date(tomorrow.getTime());
-        }
       }
     };
 
@@ -399,9 +425,6 @@
           v.endDate = new Date(v.startDate.getTime());
           v.endDate.addDays(1);
         }
-        else if (v.endDate.getTime() < tomorrow.getTime()) {
-          v.endDate = new Date(tomorrow.getTime());
-        }
       }
     };
 
@@ -413,7 +436,7 @@
         if (d.Vacation.startDateEnabled) {
           r = (!d.Vacation.endDateEnabled ||
                date.getTime() < d.Vacation.endDate.getTime()) &&
-            date.getTime() >= tomorrow.getTime();
+            date.getTime() >= today.getTime();
         }
       }
 
@@ -428,7 +451,7 @@
         if (d.Vacation.endDateEnabled) {
           r = (!d.Vacation.startDateEnabled ||
                date.getTime() > d.Vacation.startDate.getTime()) &&
-            date.getTime() >= tomorrow.getTime();
+            date.getTime() >= today.getTime();
         }
       }
 
