@@ -31,13 +31,15 @@
    * @desc The factory we'll use to register with Angular
    * @returns the Component constructor
    */
-  Component.$factory = ['$q', '$timeout', '$log', '$rootScope', 'sgSettings', 'sgComponent_STATUS', 'Preferences', 'Card', 'Gravatar', 'Resource', function($q, $timeout, $log, $rootScope, Settings, Component_STATUS, Preferences, Card, Gravatar, Resource) {
+  Component.$factory = ['$q', '$timeout', '$log', '$rootScope', 'sgSettings', 'sgComponent_STATUS', 'Preferences', 'User', 'Card', 'Gravatar', 'Resource', function($q, $timeout, $log, $rootScope, Settings, Component_STATUS, Preferences, User, Card, Gravatar, Resource) {
     angular.extend(Component, {
       STATUS: Component_STATUS,
       $q: $q,
       $timeout: $timeout,
       $log: $log,
       $rootScope: $rootScope,
+      $settings: Settings,
+      $User: User,
       $Preferences: Preferences,
       $Card: Card,
       $gravatar: Gravatar,
@@ -206,7 +208,7 @@
    * @see {@link Calendar.$getComponent}
    */
   Component.$find = function(calendarId, componentId, occurrenceId) {
-    var futureComponentData, path = [calendarId, componentId];
+    var futureComponentData, path = [calendarId, encodeURIComponent(componentId)];
 
     if (occurrenceId)
       path.push(occurrenceId);
@@ -417,12 +419,27 @@
           fields.splice(_.indexOf(fields, 'c_recurrence_id'), 1, 'occurrenceId');
 
         // Instanciate Component objects
-        _.reduce(data[type], function(components, componentData, i) {
-          var data = _.zipObject(fields, componentData), component;
-          component = new Component(data);
-          components.push(component);
-          return components;
-        }, components);
+
+        if (type == 'events') {
+          _.forEach(data[type], function(monthData, month) {
+            _.forEach(monthData.days, function(dayData, day) {
+              _.forEach(dayData.events, function(componentData, i) {
+                var data = _.zipObject(fields, componentData), component;
+                component = new Component(data);
+                dayData.events[i] = component;
+              });
+            });
+          });
+          components = data[type];
+        }
+        else if (type == 'tasks') {
+          _.reduce(data[type], function(components, componentData, i) {
+            var data = _.zipObject(fields, componentData), component;
+            component = new Component(data);
+            components.push(component);
+            return components;
+          }, components);
+        }
 
         Component.$log.debug('list of ' + type + ' ready (' + components.length + ')');
 
@@ -545,7 +562,15 @@
     else {
       this.repeat.days = [];
     }
-    if (angular.isUndefined(this.repeat.frequency))
+    if (this.repeat.dates) {
+      this.repeat.frequency = 'custom';
+      _.forEach(this.repeat.dates, function(rdate, i, rdates) {
+        if (angular.isString(rdate))
+          // Ex: 2015-10-25T22:34:51+00:00
+          rdates[i] = Component.$parseDate(rdate);
+      });
+    }
+    else if (angular.isUndefined(this.repeat.frequency))
       this.repeat.frequency = 'never';
     if (angular.isUndefined(this.repeat.interval))
       this.repeat.interval = 1;
@@ -616,6 +641,39 @@
     this.selected = false;
   };
 
+
+  /**
+   * @function initOrganizer
+   * @memberof Component.prototype
+   * @desc Extend instance with organizer including her freebusy information.
+   * @param {object} calendar - Calendar instance associated to current component
+   */
+  Component.prototype.initOrganizer = function(calendar) {
+    var _this = this, promise;
+    if (calendar && calendar.isSubscription) {
+      promise = Component.$User.$filter(calendar.owner).then(function(results) {
+        var owner = results[0];
+        _this.organizer = {
+          uid: owner.uid,
+          name: owner.cn,
+          email: owner.c_email
+        };
+      });
+    }
+    else {
+      this.organizer = {
+        uid: Component.$settings.activeUser('login'),
+        name: Component.$settings.activeUser('identification'),
+        email: Component.$settings.activeUser('email')
+      };
+      promise = Component.$q.when();
+    }
+    // Fetch organizer's freebusy
+    promise.then(function() {
+      _this.updateFreeBusyAttendee(_this.organizer);
+    });
+  };
+
   /**
    * @function hasCustomRepeat
    * @memberof Component.prototype
@@ -623,12 +681,14 @@
    * @returns true if the recurrence rule requires the full recurrence editor
    */
   Component.prototype.hasCustomRepeat = function() {
-    var b = angular.isDefined(this.repeat) &&
+    var b = angular.isUndefined(this.occurrenceId) &&
+        angular.isDefined(this.repeat) &&
         (this.repeat.interval > 1 ||
          angular.isDefined(this.repeat.days) && this.repeat.days.length > 0 ||
          angular.isDefined(this.repeat.monthdays) && this.repeat.monthdays.length > 0 ||
          angular.isDefined(this.repeat.months) && this.repeat.months.length > 0 ||
-         angular.isDefined(this.repeat.month) && angular.isDefined(this.repeat.month.type));
+         angular.isDefined(this.repeat.month) && angular.isDefined(this.repeat.month.type) ||
+         angular.isDefined(this.repeat.dates) && this.repeat.dates.length > 0);
     return b;
   };
 
@@ -769,6 +829,8 @@
     this.freebusy = this.updateFreeBusyCoverage();
 
     if (this.attendees) {
+      if (this.organizer)
+        this.updateFreeBusyAttendee(this.organizer);
       _.forEach(this.attendees, function(attendee) {
         _this.updateFreeBusyAttendee(attendee);
       });
@@ -871,9 +933,13 @@
    * @desc Add an attendee and fetch his freebusy info.
    * @param {Object} card - an Card object instance to be added to the attendees list
    */
-  Component.prototype.addAttendee = function(card) {
+  Component.prototype.addAttendee = function(card, options) {
     var _this = this, attendee, list, url, params;
     if (card) {
+      if (!this.attendees || (options && options.organizerCalendar)) {
+        // No attendee yet; initialize the organizer
+        this.initOrganizer(options? options.organizerCalendar : undefined);
+      }
       if (card.$isList({expandable: true})) {
         // Decompose list members
         list = Component.$Card.$find(card.container, card.c_name);
@@ -1044,6 +1110,31 @@
     delete this.startDate;
   };
 
+  /**
+   * @function $addRecurrenceDate
+   * @memberof Component.prototype
+   * @desc Add a start date
+   */
+  Component.prototype.$addRecurrenceDate = function() {
+    var now = new Date();
+    now.setMinutes(Math.round(now.getMinutes()/15)*15);
+
+    if (angular.isUndefined(this.repeat.dates))
+      this.repeat = { frequency: 'custom', dates: [] };
+    this.repeat.dates.push(now);
+  };
+
+  /**
+   * @function $deleteRecurrenceDate
+   * @memberof Component.prototype
+   * @desc Delete a recurrence date
+   */
+  Component.prototype.$deleteRecurrenceDate = function(index) {
+    if (index > -1 && this.repeat && this.repeat.dates && this.repeat.dates.length > index) {
+      this.repeat.dates.splice(index, 1);
+    }
+  };
+
   /**
    * @function $reset
    * @memberof Component.prototype
@@ -1067,7 +1158,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.$reply = function() {
-    var _this = this, data, path = [this.pid, this.id];
+    var _this = this, data, path = [this.pid, encodeURIComponent(this.id)];
 
     if (this.occurrenceId)
       path.push(this.occurrenceId);
@@ -1093,7 +1184,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.$adjust = function(params) {
-    var path = [this.pid, this.id];
+    var path = [this.pid, encodeURIComponent(this.id)];
 
     if (_.every(_.values(params), function(v) { return v === 0; }))
       // No changes
@@ -1130,7 +1221,7 @@
 
     // Update recurrence definition depending on selections
     if (this.hasCustomRepeat()) {
-      if (this.repeat.frequency == 'monthly' && this.repeat.month.type && this.repeat.month.type == 'byday' ||
+      if (this.repeat.frequency == 'monthly' && this.repeat.month.type && this.repeat.month.type == 'byday' && this.repeat.month.day != 'relative' ||
           this.repeat.frequency == 'yearly' && this.repeat.year.byday) {
         // BYDAY mask for a monthly or yearly recurrence
         delete component.repeat.monthdays;
@@ -1140,6 +1231,16 @@
                this.repeat.month.type) {
         // montly recurrence by month days or yearly by month
         delete component.repeat.days;
+        if (this.repeat.month.day == 'relative')
+          component.repeat.monthdays = [this.repeat.month.occurrence];
+      }
+      else if (this.repeat.frequency == 'custom' && this.repeat.dates) {
+        _.forEach(component.repeat.dates, function(rdate, i, rdates) {
+          rdates[i] = {
+            date: rdate.format(dlp, '%Y-%m-%d'),
+            time: rdate.format(dlp, '%H:%M')
+          };
+        });
       }
     }
     else if (this.repeat.frequency && this.repeat.frequency != 'never') {
@@ -1179,7 +1280,7 @@
     }
 
     // Build URL
-    path = [this.pid, this.id];
+    path = [this.pid, encodeURIComponent(this.id)];
 
     if (this.isNew)
       options = { action: 'saveAs' + this.type.capitalize() };
@@ -1204,7 +1305,7 @@
    * @param {boolean} occurrenceOnly - delete this occurrence only
    */
   Component.prototype.remove = function(occurrenceOnly) {
-    var _this = this, path = [this.pid, this.id];
+    var _this = this, path = [this.pid, encodeURIComponent(this.id)];
 
     if (occurrenceOnly && this.occurrenceId)
       path.push(this.occurrenceId);
@@ -1278,13 +1379,16 @@
   Component.prototype.alarmDescription = function() {
     var key, localizedString = null;
     if (this.alarm) {
-      key = ['reminder' + this.alarm.quantity, this.alarm.unit, this.alarm.reference].join('_');
+      key = ['reminder', this.alarm.quantity];
+      if (this.alarm.quantity > 0)
+        key.push(this.alarm.unit.toUpperCase(), this.alarm.reference.toUpperCase());
+      key = key.join('_');
       localizedString = l(key);
       if (key === localizedString)
         // No localized string for this reminder definition
         localizedString = [this.alarm.quantity,
-                           l('reminder_' + this.alarm.unit),
-                           l('reminder_' + this.alarm.reference)].join(' ');
+                           l('reminder_' + this.alarm.unit.toUpperCase()),
+                           l('reminder_' + this.alarm.reference.toUpperCase())].join(' ');
     }
 
     return localizedString;
@@ -1298,7 +1402,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.copyTo = function(calendar) {
-    return Component.$$resource.post(this.pid + '/' + this.id, 'copy', {destination: calendar});
+    return Component.$$resource.post(this.pid + '/' + encodeURIComponent(this.id), 'copy', {destination: calendar});
   };
 
   /**
@@ -1309,7 +1413,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.moveTo = function(calendar) {
-    return Component.$$resource.post(this.pid + '/' + this.id, 'move', {destination: calendar});
+    return Component.$$resource.post(this.pid + '/' + encodeURIComponent(this.id), 'move', {destination: calendar});
   };
 
   Component.prototype.toString = function() {
