@@ -222,11 +222,15 @@
     vm.showAttendeesEditor = vm.component.attendees && vm.component.attendees.length;
     vm.toggleAttendeesEditor = toggleAttendeesEditor;
     //vm.searchText = null;
+    vm.changeFrequency = changeFrequency;
+    vm.changeCalendar = changeCalendar;
     vm.cardFilter = cardFilter;
     vm.addAttendee = addAttendee;
     vm.removeAttendee = removeAttendee;
     vm.addAttachUrl = addAttachUrl;
     vm.priorityLevel = priorityLevel;
+    vm.changeAlarmRelation = changeAlarmRelation;
+    vm.onAlarmChange = onAlarmChange;
     vm.reset = reset;
     vm.cancel = cancel;
     vm.edit = edit;
@@ -237,7 +241,9 @@
       hours: getHours()
     };
     vm.addStartDate = addStartDate;
+    vm.removeStartDate = removeStartDate;
     vm.addDueDate = addDueDate;
+    vm.removeDueDate = removeDueDate;
 
     // Synchronize start and end dates
     vm.adjustStartTime = adjustStartTime;
@@ -271,6 +277,17 @@
         vm.component.repeat.month.type == 'bymonthday';
     }
 
+    function changeFrequency() {
+      if (vm.component.repeat.frequency == 'custom')
+        vm.showRecurrenceEditor = true;
+    }
+
+    function changeCalendar() {
+      var updateRequired = (vm.component.attendees && vm.component.attendees.length > 0);
+      if (updateRequired)
+        vm.component.initOrganizer(Calendar.$get(vm.component.destinationCalendar));
+    }
+
     // Autocomplete cards for attendees
     function cardFilter($query) {
       AddressBook.$filterAll($query);
@@ -278,18 +295,47 @@
     }
 
     function addAttendee(card) {
-      var automaticallyExapand = (!vm.component.attendees || vm.component.attendees.length === 0);
+      var initOrganizer = (!vm.component.attendees || vm.component.attendees.length === 0),
+          destinationCalendar = Calendar.$get(vm.component.destinationCalendar),
+          options = initOrganizer? { organizerCalendar: destinationCalendar } : {};
+      var emailRE = /([\w\!\#$\%\&\'\*\+\-\/\=\?\^\`{\|\}\~]+\.)*[\w\!\#$\%\&\'\*\+\-\/\=\?\^\`{\|\}\~]+@((((([a-z0-9]{1}[a-z0-9\-]{0,62}[a-z0-9]{1})|[a-z])\.)+[a-z]{2,})|(\d{1,3}\.){3}\d{1,3}(\:\d{1,5})?)/i,
+          i, address;
+
+      function createCard(str) {
+        var match = str.match(emailRE),
+            email = match[0],
+            name = str.replace(new RegExp(" *<?" + email + ">? *"), '');
+        vm.showAttendeesEditor |= initOrganizer;
+        vm.searchText = '';
+        return new Card({ c_cn: _.trim(name, ' "'), emails: [{ value: email }] });
+      }
+
       if (angular.isString(card)) {
         // User pressed "Enter" in search field, adding a non-matching card
-        if (card.isValidEmail()) {
-          vm.component.addAttendee(new Card({ emails: [{ value: card }] }));
-          vm.showAttendeesEditor |= automaticallyExapand;
-          vm.searchText = '';
+        // Examples that are handled:
+        //   Smith, John <john@smith.com>
+        //   <john@appleseed.com>;<foo@bar.com>
+        //   foo@bar.com abc@xyz.com
+        address = '';
+        for (i = 0; i < card.length; i++) {
+          if ((card.charCodeAt(i) ==  9 ||   // tab
+               card.charCodeAt(i) == 32 ||   // space
+               card.charCodeAt(i) == 44 ||   // ,
+               card.charCodeAt(i) == 59) &&  // ;
+              emailRE.test(address)) {
+            vm.component.addAttendee(createCard(address), options);
+            address = '';
+          }
+          else {
+            address += card.charAt(i);
+          }
         }
+        if (address)
+          vm.component.addAttendee(createCard(address), options);
       }
       else {
-        vm.component.addAttendee(card);
-        vm.showAttendeesEditor |= automaticallyExapand;
+        vm.component.addAttendee(card, options);
+        vm.showAttendeesEditor |= initOrganizer;
       }
     }
 
@@ -311,7 +357,31 @@
       }
     }
 
+    function changeAlarmRelation(form) {
+      if (vm.component.type == 'task' && vm.component.$hasAlarm &&
+          ((!vm.component.start && vm.component.alarm.relation == 'START') ||
+           (!vm.component.due   && vm.component.alarm.relation == 'END'))) {
+        form.alarmRelation.$setValidity('alarm', false);
+      }
+      else {
+        form.alarmRelation.$setValidity('alarm', true);
+      }
+    }
+
+    function onAlarmChange(form) {
+      if (vm.component.type !== 'task') {
+        return;
+      }
+      if (!vm.component.start && vm.component.alarm.relation == 'START') {
+        vm.component.alarm.relation = 'END';
+      } else if (!vm.component.due && vm.component.alarm.relation == 'END') {
+        vm.component.alarm.relation = 'START';
+      }
+      changeAlarmRelation(form);
+    }
+
     function save(form, options) {
+      changeAlarmRelation(form);
       if (form.$valid) {
         vm.component.$save(options)
           .then(function(data) {
@@ -339,7 +409,7 @@
         // Cancelling the creation of a component
         vm.component = null;
       }
-      $mdDialog.cancel();
+      $mdDialog.hide();
     }
 
     function edit(form) {
@@ -368,14 +438,38 @@
       return hours;
     }
 
-    function addStartDate() {
+    function addStartDate(form) {
       vm.component.$addStartDate();
       oldStartDate = new Date(vm.component.start.getTime());
+      if (!vm.component.due) {
+        vm.component.alarm.relation = 'START';
+      }
+      changeAlarmRelation(form);
     }
 
-    function addDueDate() {
+    function removeStartDate(form) {
+      vm.component.$deleteStartDate();
+      if (vm.component.due) {
+        vm.component.alarm.relation = 'END';
+      }
+      changeAlarmRelation(form);
+    }
+
+    function addDueDate(form) {
       vm.component.$addDueDate();
       oldDueDate = new Date(vm.component.due.getTime());
+      if (!vm.component.start) {
+        vm.component.alarm.relation = 'END';
+      }
+      changeAlarmRelation(form);
+    }
+
+    function removeDueDate(form) {
+      vm.component.$deleteDueDate();
+      if (vm.component.start) {
+        vm.component.alarm.relation = 'START';
+      }
+      changeAlarmRelation(form);
     }
 
     function adjustStartTime() {
